/**************************************************************************//**
 * @file     GPIO_Monitor_Dlg.cpp
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 13/12/03 11:27a $
 * @brief    NuBridge I2C Monitor Main Dialog
 *
 * @note
 * Copyright (C) 2013 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
// GPIO_Monitor_Dlg.cpp : implementation file
//

#include "stdafx.h"
#include "NuBridge.h"
#include "GPIO_Monitor_Dlg.h"
#include "NuDlgMessage.h"


// GPIO_Monitor_Dlg dialog
UCHAR g_u8DataFilterR = 0, g_u8DataFilterW = 0, g_u8DataFilterACK = 0, g_u8DataFilterNACK = 0;
UCHAR g_u8DeviceFilterAll = 0, g_u8DeviceFilterDevice = 0;
UCHAR g_u8DeviceFilterAdd = 0;
 
IMPLEMENT_DYNAMIC(GPIO_Monitor_Dlg, CDialog)

GPIO_Monitor_Dlg::GPIO_Monitor_Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(GPIO_Monitor_Dlg::IDD, pParent)
{
	m_bInit = FALSE;
	m_CtrlCnt = 0;
}

GPIO_Monitor_Dlg::~GPIO_Monitor_Dlg()
{
}

void GPIO_Monitor_Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_EDIT_GPIO_MONITOR_ADDRESS, m_EDIT_I2CM_ADDRESS);

	DDX_Control(pDX, IDC_BTN_GPIO_MONITOR_START, m_I2CMbtns[0]);
	DDX_Control(pDX, IDC_BTN_GPIO_MONITOR_STOP, m_I2CMbtns[1]);
	DDX_Control(pDX, IDC_CHECK_DFREAD, m_CHECK_Read);
	DDX_Control(pDX, IDC_CHECK_DFWRITE, m_CHECK_Write);
	DDX_Control(pDX, IDC_CHECK_DFACK, m_CHECK_ACK);
	DDX_Control(pDX, IDC_CHECK_DFNACK, m_CHECK_NACK);
	DDX_Control(pDX, IDC_RADIO_GPIO_MONITOR_ALL, m_RADIODevice[0]);
	DDX_Control(pDX, IDC_RADIO_GPIO_MONITOR_FILTER, m_RADIODevice[1]);
}


BEGIN_MESSAGE_MAP(GPIO_Monitor_Dlg, CDialog)
	ON_BN_CLICKED(IDC_BTN_GPIO_MONITOR_START, &GPIO_Monitor_Dlg::OnBnClickedBtnGpioMonitorStart)
	ON_BN_CLICKED(IDC_BTN_GPIO_MONITOR_STOP, &GPIO_Monitor_Dlg::OnBnClickedBtnGpioMonitorStop)

	ON_NOTIFY_EX(TTN_NEEDTEXT, 0, &GPIO_Monitor_Dlg::SetTipText)
END_MESSAGE_MAP()


// GPIO_Monitor_Dlg message handlers

BOOL GPIO_Monitor_Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// TODO:  Add extra initialization here
	InitDialogWnd(FALSE);
	EnableToolTips(TRUE);

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}
void GPIO_Monitor_Dlg::I2CMSwitchLang(HINSTANCE tmpDll, int LanguageIdx)
{
	CString text;
	int i;

	if(tmpDll && (LanguageIdx > 0))
	{
		text.LoadString(tmpDll, IDS_DATAFILTER);
		GetDlgItem(IDC_STATIC_GPIO_MONITOR_GROUP)->SetWindowText(text);

		text.LoadString(tmpDll, IDS_SHOWALL);
		GetDlgItem(IDC_RADIO_GPIO_MONITOR_ALL)->SetWindowText(text);

		text.LoadString(tmpDll, IDS_DEVICEADD);
		GetDlgItem(IDC_RADIO_GPIO_MONITOR_FILTER)->SetWindowText(text);
	}

	if(LanguageIdx == 1)
	{
		i = 0;
		m_I2CMbtns[i].SetBitmaps(IDB_BITMAP_CHT_BTN_MODE, RGB(0, 255, 0)); 
		m_I2CMbtns[i++].DrawTransparent();

		m_I2CMbtns[i].SetBitmaps(IDB_BITMAP_CHT_BTN_STOP, RGB(0, 255, 0)); 
		m_I2CMbtns[i++].DrawTransparent();
	}
	else if(LanguageIdx == 2)
	{
		i = 0;
		m_I2CMbtns[i].SetBitmaps(IDB_BITMAP_CHS_BTN_MODE, RGB(0, 255, 0)); 
		m_I2CMbtns[i++].DrawTransparent();

		m_I2CMbtns[i].SetBitmaps(IDB_BITMAP_CHS_BTN_STOP, RGB(0, 255, 0)); 
		m_I2CMbtns[i++].DrawTransparent();
	}
	else    // default language(English)
	{
		i = 0;
		m_I2CMbtns[i].SetBitmaps(IDB_BITMAP_BTN_MODE, RGB(0, 255, 0)); 
		m_I2CMbtns[i++].DrawTransparent();

		m_I2CMbtns[i].SetBitmaps(IDB_BITMAP_BTN_STOP, RGB(0, 255, 0)); 
		m_I2CMbtns[i++].DrawTransparent();

		GetDlgItem(IDC_STATIC_GPIO_MONITOR_GROUP)->SetWindowText(_T("Device Filter"));
		GetDlgItem(IDC_RADIO_GPIO_MONITOR_ALL)->SetWindowText(_T("Show All"));
		GetDlgItem(IDC_RADIO_GPIO_MONITOR_FILTER)->SetWindowText(_T("Device Address 0x"));
	}
	LangDll = tmpDll;
}
void GPIO_Monitor_Dlg::OnBnClickedBtnGpioMonitorStart()
{
	// TODO: Add your control notification handler code here
	if(NuUsb.WinUsbNumber > 0)
	{
		GetParent()->SendMessage(WM_LOCK_SCREEN, (WPARAM)1);	
		InitDialogWnd(FALSE);
	}
	else
	{
		AfxMessageBox(_T("Nu-Bridge Not Found"));
		InitDialogWnd(FALSE);
		return;
	}
}

void GPIO_Monitor_Dlg::OnBnClickedBtnGpioMonitorStop()
{
	// TODO: Add your control notification handler code here
	
	if(NuUsb.WinUsbNumber > 0)
	{
		GetParent()->SendMessage(WM_LOCK_SCREEN, (WPARAM)0);	
		InitDialogWnd(TRUE);
	}
	else
	{
		AfxMessageBox(_T("Nu-Bridge Not Found"));
		InitDialogWnd(FALSE);
		return;
	}
}

void GPIO_Monitor_Dlg::InitDialogWnd(bool bShow)
{
	// Disable All Member
	if(!m_bInit)
	{
		int i = 0;
		m_I2CMbtns[i].SetBitmaps(IDB_BITMAP_BTN_MODE, RGB(0, 255, 0)); 
		m_I2CMbtns[i++].DrawTransparent();

		m_I2CMbtns[i].SetBitmaps(IDB_BITMAP_BTN_STOP, RGB(0, 255, 0)); 
		m_I2CMbtns[i++].DrawTransparent();

		m_bInit = TRUE;
		m_CtrlCnt = i;

		m_RADIODevice[0].SetCheck(1);
	}

	if(NuUsb.WinUsbNumber == 0)
	{
		for(int i = 0; i < m_CtrlCnt; i++)
		{
			m_I2CMbtns[i].EnableWindow(FALSE);
		}
	}
	else
	{
		m_I2CMbtns[0].EnableWindow(bShow);
		for(int i = 1; i < m_CtrlCnt; i++)
		{
			m_I2CMbtns[i].EnableWindow(!bShow);
		}
	}
} 


void GPIO_Monitor_Dlg::OnOK()
{
	// TODO: Add your specialized code here and/or call the base class

	//CDialog::OnOK();
}

void GPIO_Monitor_Dlg::OnCancel()
{
	// TODO: Add your specialized code here and/or call the base class

	//CDialog::OnCancel();
}

BOOL GPIO_Monitor_Dlg::SetTipText(UINT id, NMHDR *pTTTStruct, LRESULT *pResult)
{
	TOOLTIPTEXT *pTTT = (TOOLTIPTEXT *)pTTTStruct;          
	UINT nID = pTTTStruct->idFrom; 

	if (pTTT->uFlags & TTF_IDISHWND)  
	{
		nID = ::GetDlgCtrlID((HWND)nID);
		if (NULL == nID)
			return FALSE;
		
		switch(nID)
		{				
			case IDC_BTN_GPIO_MONITOR_START:
			{
				if(LangDll == NULL)
					pTTT->lpszText = _T("Start I2C Monitor.");
				else
				{
					ToolTipStr.LoadString(LangDll, IDS_TOOLTIP_I2CM_MODE);
					pTTT->lpszText = (LPWSTR)((LPCTSTR)ToolTipStr);
				}
			}
			break;
			
			case IDC_BTN_GPIO_MONITOR_STOP:
			{
				if(LangDll == NULL)
					pTTT->lpszText = _T("Stop I2C Monitor.");
				else
				{
					ToolTipStr.LoadString(LangDll, IDS_TOOLTIP_I2CM_DISABLE);
					pTTT->lpszText = (LPWSTR)((LPCTSTR)ToolTipStr);
				}
			}	
			break;

			default:
				break;
		}

		return TRUE;
	}  
	return FALSE;
}
