/**************************************************************************//**
 * @file     NuBridgeDlg.cpp
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 13/12/03 11:27a $
 * @brief    NuBridge Main Dialog
 *
 * @note
 * Copyright (C) 2013 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
// NuBridgeDlg.cpp : implementation file
//

#include "stdafx.h"
#include "NuBridge.h"
#include "NuBridgeDlg.h"
#include "Winnt.h"
#include "PWM_Capture_Dlg.h"
#include <windows.h>
#include <dbt.h>
#include <Setupapi.h>

#include "NuWidget.h"

static const GUID GUID_DEVINTERFACE_LIST[] = 
{
	{0x1f4911f2, 0xc55d, 0x4561, {0x98, 0x7a, 0xbc, 0xfa, 0x74, 0xd5, 0xbb, 0x6d}},

};

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CNuBridgeDlg dialog
extern CString strSPI0Script; 
extern int cmdCnt;

extern volatile int UsbBufTail;
extern volatile int UsbBufHead;


CFont cf_main;
int l_FontHeight_main = 16;
static BOOL bActiveUSB = 0;

int m_iLanguage;

CNuBridgeDlg::CNuBridgeDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CNuBridgeDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CNuBridgeDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_STACK_FRM, m_MainFrameWnd);

	DDX_Control(pDX, IDC_MAIN_TAB, m_tabCtrl);
	DDX_Control(pDX, IDC_MAIN_LIST, m_MAIN_LIST);

	DDX_Control(pDX, IDC_BUTTON_CONNECT, m_btnButtons[0]); //connect button
	DDX_Control(pDX, IDC_CLEAR, m_btnButtons[1]);		   //clear   button
	DDX_Control(pDX, IDC_STATIC_LOGO, m_LOGO);
    DDX_Control(pDX, IDC_RADIO_D1, m_RADIO_Device[0]);
	DDX_Control(pDX, IDC_RADIO_D2, m_RADIO_Device[1]);
	DDX_Control(pDX, IDC_RADIO_D3, m_RADIO_Device[2]);
	DDX_Control(pDX, IDC_RADIO_D4, m_RADIO_Device[3]);
}

BEGIN_MESSAGE_MAP(CNuBridgeDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_BUTTON_CONNECT, &CNuBridgeDlg::OnBnClickedButtonConnect)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_BTN_USB_ACTIVE, &CNuBridgeDlg::OnBnClickedBtnUsbActive)
	ON_COMMAND(ID_LANGUAGE_US, &CNuBridgeDlg::OnLanguageUs)
	ON_COMMAND(ID_LANGUAGE_CHINESETRADITIONAL, &CNuBridgeDlg::OnLanguageChinesetraditional)
	ON_COMMAND(ID_LANGUAGE_CHINESESIMPLE, &CNuBridgeDlg::OnLanguageChinesesimple)
	ON_BN_CLICKED(IDC_RADIO_D1, &CNuBridgeDlg::OnBnClickedRadioD1)
	ON_BN_CLICKED(IDC_RADIO_D2, &CNuBridgeDlg::OnBnClickedRadioD2)
	ON_BN_CLICKED(IDC_RADIO_D3, &CNuBridgeDlg::OnBnClickedRadioD3)
	ON_BN_CLICKED(IDC_RADIO_D4, &CNuBridgeDlg::OnBnClickedRadioD4)
	ON_BN_CLICKED(IDC_CLEAR, &CNuBridgeDlg::OnBnClickedClear)

	ON_MESSAGE(WM_TAB_PAGE_CLICKED, &CNuBridgeDlg::OnTabClicked)
	ON_MESSAGE(WM_LOCK_SCREEN, &CNuBridgeDlg::LockScreen)
	ON_MESSAGE(WM_LOCK_LISTCTRL, &CNuBridgeDlg::LockListCtrl)
	ON_MESSAGE(WM_DEVICECHANGE, &CNuBridgeDlg::OnMyDeviceChange)

	ON_NOTIFY(TCN_SELCHANGE, IDC_MAIN_TAB, &CNuBridgeDlg::OnTcnSelchangeMainTab)
	ON_NOTIFY_EX(TTN_NEEDTEXT, 0, &CNuBridgeDlg::SetTipText)
END_MESSAGE_MAP()


// CNuBridgeDlg message handlers

BOOL CNuBridgeDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here

	//default language (English)
	m_iLanguage = 0;

	m_Menu.LoadMenu(IDR_LANGUAGE_MENU);
	SetMenu(&m_Menu);
	
	m_MainFrameWnd.CreateStackedWnd(0);
	
	// DUMMY TAB ITEM
	m_tabCtrl.Init();

	//default language (English)
	SwitchLang(NULL);
	
	HBITMAP hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_LOGO), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
	m_LOGO.SetBitmap(hbmp);
	
	m_MAIN_LIST.SetExtendedStyle(LVS_EX_FLATSB  | LVS_EX_HEADERDRAGDROP | LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_NOSCROLL);	

	CString csFontPath = _T("Courier new Bold.ttf"); 
	AddFontResource(csFontPath);
	cf_main.CreateFont(l_FontHeight_main,0,0,0,FW_NORMAL,FALSE, FALSE, 
					   FALSE,DEFAULT_CHARSET ,OUT_DEFAULT_PRECIS,
				       CLIP_DEFAULT_PRECIS,ANTIALIASED_QUALITY,
                       DEFAULT_PITCH, _T("Courier new Bold"));    
	m_MAIN_LIST.SetFont(&cf_main,true);
		

	LV_COLUMN Col;
	CRect r_list;
	m_MAIN_LIST.GetWindowRect(&r_list);
	ScreenToClient(r_list);
	
	Col.mask = LVCF_FMT|LVCF_SUBITEM|LVCF_TEXT|LVCF_WIDTH; 
	Col.fmt = LVCFMT_LEFT;
	Col.iSubItem = 0;
	Col.cx = 80; 
	Col.pszText = _T("# ");
	m_MAIN_LIST.InsertColumn(0, &Col);

	Col.iSubItem = 1;
	Col.cx = r_list.Width() * 4; 
	Col.pszText = _T("Data");
	m_MAIN_LIST.InsertColumn(1, &Col);

	GetDlgItem(IDC_BTN_USB_ACTIVE)->ShowWindow(FALSE); 
	ShowMainFrameRadio(FALSE, 4);


	HDEVNOTIFY hDevNotify;
    DEV_BROADCAST_DEVICEINTERFACE NotificationFilter;
    ZeroMemory( &NotificationFilter, sizeof(NotificationFilter) );
    NotificationFilter.dbcc_size = sizeof(DEV_BROADCAST_DEVICEINTERFACE);
    NotificationFilter.dbcc_devicetype = DBT_DEVTYP_DEVICEINTERFACE;
	
	NotificationFilter.dbcc_classguid = GUID_DEVINTERFACE_LIST[0];
	hDevNotify = RegisterDeviceNotification(this->GetSafeHwnd(), &NotificationFilter, DEVICE_NOTIFY_WINDOW_HANDLE);
	if(!hDevNotify) 
	{
		AfxMessageBox(_T("Can't register device notification"));
		return 0;
	}

	EnableToolTips(TRUE);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CNuBridgeDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CNuBridgeDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}


void CNuBridgeDlg::OnOK()
{
	// TODO: Add your specialized code here and/or call the base class

	//CDialog::OnOK();
}

void CNuBridgeDlg::OnCancel()
{
	// TODO: Add your specialized code here and/or call the base class

	//CDialog::OnCancel();
}

void CNuBridgeDlg::OnClose()
{
	// TODO: Add your message handler code here and/or call default
	CDialog::OnClose();

	if(NuUsb.WinUsbNumber > 0)
	{
		EndUsbMonitor();
		NuUsb.NU_ResetFW();
	}


	if (hDLL!=NULL)
	{
		FreeLibrary(hDLL);
		hDLL= NULL;
	}
	CDialog::OnOK();
	
}
void CNuBridgeDlg::OnBnClickedButtonConnect()
{
	// TODO: Add your control notification handler code here
	ShowMainFrameRadio(FALSE, 4);  
	DeviceRadioCheck(FALSE);
	bActiveUSB = 0;

	if(NuUsb.WinUsbNumber)
	{		
		EndUsbMonitor();
		NuUsb.NU_CloseHandle();
		NuUsb.EnableWinUsbDevice();		
	}
	else
	{
		NuUsb.EnableWinUsbDevice();		
	}

	if(NuUsb.WinUsbNumber == 0)
	{
		MessageBox(_T("Nu-Bridge Not Found"));
		GetDlgItem(IDC_BTN_USB_ACTIVE)->ShowWindow(FALSE);
		EndUsbMonitor();		
	}
	else
	{
		//CString devFound;
		int devCnt = NuUsb.WinUsbNumber;
		//devFound.Format(_T("Found %d Device"), devCnt);
		//MessageBox(devFound);

		GetDlgItem(IDC_BTN_USB_ACTIVE)->ShowWindow(TRUE);
		if(bActiveUSB)
		{
			GetDlgItem(IDC_BTN_USB_ACTIVE)->SetWindowText(_T("Deactive"));
		}
		else
		{
			GetDlgItem(IDC_BTN_USB_ACTIVE)->SetWindowText(_T("Active"));
		}

		GetDlgItem(IDC_BTN_USB_ACTIVE)->EnableWindow(TRUE);

		ShowMainFrameRadio(TRUE, devCnt);

		memset(&RadioValid[0], 0 , sizeof(BOOL)*4);

		NuUsb.m_ActiveDevice = 0;
		m_RADIO_Device[0].SetCheck(true);

		SwitchUsbDevice();
		m_tabCtrl.FunModuleDlg();
	}

}

void CNuBridgeDlg::StartUsbMonitor(void)
{
	if(g_bThreadContinue)
		return;

	m_pUsbReadThread = AfxBeginThread(UsbReadThread, (LPVOID)0);
	m_pListWriteThread = AfxBeginThread(ListWriteThread, (LPVOID)this);
}

void CNuBridgeDlg::EndUsbMonitor()
{
	if(g_bThreadContinue)
	{
		g_bThreadContinue = FALSE;
		int WorkDev = NuUsb.m_WorkDevice;
		NuUsb.NU_AbortReadPipe(WorkDev);

	
		if(::WaitForSingleObject(m_pUsbReadThread->m_hThread, 10) == WAIT_TIMEOUT)
			TerminateThread(m_pUsbReadThread->m_hThread, 0);

		if(::WaitForSingleObject(m_pListWriteThread->m_hThread,10) == WAIT_TIMEOUT)
			TerminateThread(m_pListWriteThread->m_hThread, 0);

		m_pUsbReadThread = NULL;
		m_pListWriteThread = NULL;
	}
}

int CNuBridgeDlg::SwitchUsbDevice()
{
	EndUsbMonitor();
	StartUsbMonitor();
	return m_DeviceId;
}

void CNuBridgeDlg::OnTcnSelchangeMainTab(NMHDR *pNMHDR, LRESULT *pResult)
{
	// TODO: Add your control notification handler code here

	// Do something with the "formal parameters" so the compiler is happy in warning level 4
	pNMHDR = NULL;
	pResult = NULL;
}
void CNuBridgeDlg::ShowMainFrameRadio(BOOL bShow, UINT RadioNum)
{
	for(UINT i = 0; i < RadioNum; i++)
	{
		m_RADIO_Device[i].ShowWindow(bShow);
		m_RADIO_Device[i].EnableWindow(bShow);
	}
}

void CNuBridgeDlg::DeviceRadioCheck(BOOL bShow)
{
	for(int i = 0; i < 4; i++)
	{
		m_RADIO_Device[i].SetCheck(bShow);
	}
}

LRESULT CNuBridgeDlg::OnTabClicked(WPARAM wParam, LPARAM lParam)
{	
	// TODO: Add your control notification handler code here

	OnBnClickedClear();
	return 1;
}

LRESULT CNuBridgeDlg::LockScreen(WPARAM wParam, LPARAM lParam)
{
	BOOL bLock = (BOOL)(wParam);
	
	// Lock
	if(bLock)
	{
		// Lock Main Dialog Button
		GetDlgItem(IDC_BUTTON_CONNECT)->EnableWindow(FALSE);
		GetDlgItem(IDC_BTN_USB_ACTIVE)->EnableWindow(FALSE);

		if(!bActiveUSB)
		{
			OnBnClickedBtnUsbActive();
		}	
	}
	else
	{
		// UnLock Main Dialog Button
		GetDlgItem(IDC_BUTTON_CONNECT)->EnableWindow(TRUE);
		GetDlgItem(IDC_BTN_USB_ACTIVE)->EnableWindow(TRUE);		
	}

	return bLock;
}
void CNuBridgeDlg::OnTimer(UINT_PTR nIDEvent)
{
	// TODO: Add your message handler code here and/or call default
	CString strItem;

	KillTimer(nIDEvent);
	switch(nIDEvent)
	{
		// Warning Message is processed by NuBridgeDlg
		case MODE_SPI_MONITOR:
				MessageBox(_T("SS Buffer is full, Please restart to get data."));
		break;

		case MODE_PWM:
				MessageBox(_T("Capture Buffer is full, Please restart to get data."));
				((PWM_Capture_Dlg*)(m_tabCtrl.m_tabPages[4]))->OnBnClickedBtnPwmcDisable();
				
		break;
		
		// String display is processed by NuWidget
		case MODE_I2C_MONITOR:
		case 0xFF:
			TimeOut(nIDEvent);
		break;
		
		default:
		break;
	}

	CDialog::OnTimer(nIDEvent);
}

void CNuBridgeDlg::OnBnClickedBtnUsbActive()
{
	// TODO: Add your control notification handler code here
	if(!bActiveUSB)
	{
		GetDlgItem(IDC_BTN_USB_ACTIVE)->SetWindowText(_T("Deactive"));
		for(int i = 0; i < NuUsb.WinUsbNumber; i++)
		{
			m_RADIO_Device[i].ShowWindow(TRUE);
			m_RADIO_Device[i].EnableWindow(FALSE);
		}

		if((NuUsb.m_ActiveDevice != NuUsb.m_WorkDevice) && g_bThreadContinue)
			SwitchUsbDevice();
			

	}
	else
	{
		GetDlgItem(IDC_BTN_USB_ACTIVE)->SetWindowText(_T("Active"));
		for(int i = 0; i < NuUsb.WinUsbNumber; i++)
		{
			if(RadioValid[i] == 1)
				m_RADIO_Device[i].EnableWindow(FALSE);
			else
				m_RADIO_Device[i].EnableWindow(TRUE);
		}
	}

	// TODO: Add your control notification handler code here
	bActiveUSB = !(bActiveUSB);
}

void CNuBridgeDlg::SwitchLang(HINSTANCE tmpDll)
{
	CString text;
    int i;

	if(tmpDll && (m_iLanguage > 0))
	{
		text.LoadString(tmpDll, IDS_DEVICE1);
		m_RADIO_Device[0].SetWindowText(text);

		text.LoadString(tmpDll, IDS_DEVICE2);
		m_RADIO_Device[1].SetWindowText(text);

		text.LoadString(tmpDll, IDS_DEVICE3);
		m_RADIO_Device[2].SetWindowText(text);

		text.LoadString(tmpDll, IDS_DEVICE4);
		m_RADIO_Device[3].SetWindowText(text);
	}

	if(m_iLanguage == 0)	 // default language(English)
	{
        i = 0;
		m_btnButtons[i].SetBitmaps(IDB_BITMP_LEFT_CONNECT, RGB(0, 255, 0));
		m_btnButtons[i++].DrawTransparent();
		m_btnButtons[i].SetBitmaps(IDB_BITMAP_BTN_CLR, RGB(0, 255, 0));
		m_btnButtons[i++].DrawTransparent();

		m_RADIO_Device[0].SetWindowText(_T("Device 1"));
		m_RADIO_Device[1].SetWindowText(_T("Device 2"));
		m_RADIO_Device[2].SetWindowText(_T("Device 3"));
		m_RADIO_Device[3].SetWindowText(_T("Device 4"));
	}
	else if(m_iLanguage == 1)
	{
        i = 0;
		m_btnButtons[i].SetBitmaps(IDB_BITMP_CHT_LEFT_CONNECT, RGB(0, 255, 0));
		m_btnButtons[i++].DrawTransparent();
		m_btnButtons[i].SetBitmaps(IDB_BITMAP_CHT_BTN_CLR, RGB(0, 255, 0));
		m_btnButtons[i++].DrawTransparent();
	}
	else
	{
        i = 0;
		m_btnButtons[i].SetBitmaps(IDB_BITMP_CHS_LEFT_CONNECT, RGB(0, 255, 0));
		m_btnButtons[i++].DrawTransparent();
		m_btnButtons[i].SetBitmaps(IDB_BITMAP_CHS_BTN_CLR, RGB(0, 255, 0));
		m_btnButtons[i++].DrawTransparent();

	}

	m_tabCtrl.SwitchLang(hDLL, m_iLanguage);
	m_MAIN_LIST.m_hLangDLL = hDLL;
}

void CNuBridgeDlg::OnLanguageUs()
{
	// TODO: Add your command handler code here
	CString text;
	CMenu *menu    = new CMenu();
	menu->CreateMenu();

	//another menu object for submenu
	CMenu *subMenu = new CMenu();
	subMenu->CreatePopupMenu();

	if (hDLL != NULL)
	{
		FreeLibrary(hDLL);
		hDLL= NULL;
	}

	hDLL = LoadLibrary(_T("NuLang_en-US.dll"));
	if ( hDLL )
    {	
		m_iLanguage = 0;

		text.LoadString(hDLL, IDS_ENG);
		subMenu->AppendMenu(MF_STRING, ID_LANGUAGE_US, text); 
		text.LoadString(hDLL, IDS_CHT);
		subMenu->AppendMenu(MF_STRING, ID_LANGUAGE_CHINESETRADITIONAL, text); 
		text.LoadString(hDLL, IDS_CHS);
		subMenu->AppendMenu(MF_STRING, ID_LANGUAGE_CHINESESIMPLE, text); 
		//append submenu to menu
		text.LoadString(hDLL, IDS_LANGUAGE);
		menu->AppendMenu(MF_POPUP|MF_STRING, (UINT)subMenu->m_hMenu, text); 
		

		SetMenu(menu);

		SwitchLang(hDLL);
    }
	else
	{
		m_iLanguage = 0;
		hDLL= NULL;
	}

	if(menu)
	{
		menu->DestroyMenu();
		delete menu;
	}

	if(subMenu)
	{
		subMenu->DestroyMenu();
		delete subMenu;
	}
}

void CNuBridgeDlg::OnLanguageChinesetraditional()
{
	// TODO: Add your command handler code here
	CString text;
	CMenu *menu1    = new CMenu();
	menu1->CreateMenu();

	//another menu object for submenu
	CMenu *subMenu1 = new CMenu();
	subMenu1->CreatePopupMenu();

	if (hDLL != NULL)
	{
		FreeLibrary(hDLL);
		hDLL= NULL;
	}

	hDLL = LoadLibrary(_T("NuLang_zh-CHT.dll"));
	if ( hDLL )
    {
		m_iLanguage = 1;
		text.LoadString(hDLL, IDS_ENG);
		subMenu1->AppendMenu(MF_STRING, ID_LANGUAGE_US, text); 
		text.LoadString(hDLL, IDS_CHT);
		subMenu1->AppendMenu(MF_STRING, ID_LANGUAGE_CHINESETRADITIONAL, text); 
		text.LoadString(hDLL, IDS_CHS);
		subMenu1->AppendMenu(MF_STRING, ID_LANGUAGE_CHINESESIMPLE, text); 

		//append submenu to menu
		text.LoadString(hDLL, IDS_LANGUAGE);
		menu1->AppendMenu(MF_POPUP|MF_STRING, (UINT)subMenu1->m_hMenu, text); 
		SetMenu(menu1);

		SwitchLang(hDLL);
    }
	else
	{
		m_iLanguage = 0;
		hDLL = NULL;
	}

	if(menu1)
	{
		menu1->DestroyMenu();
		delete menu1;
	}

	if(subMenu1)
	{
		subMenu1->DestroyMenu();
		delete subMenu1;
	}

}

void CNuBridgeDlg::OnLanguageChinesesimple()
{
	// TODO: Add your command handler code here
	CString text;
	CMenu *menu    = new CMenu();
	menu->CreateMenu();

	//another menu object for submenu
	CMenu *subMenu = new CMenu();
	subMenu->CreatePopupMenu();

	if (hDLL != NULL)
	{
		FreeLibrary(hDLL);
		hDLL= NULL;
	}

	hDLL = LoadLibrary(_T("NuLang_zh-CHS.dll"));

	if ( hDLL )
    {
		m_iLanguage = 2;
		text.LoadString(hDLL, IDS_ENG);
		subMenu->AppendMenu(MF_STRING, ID_LANGUAGE_US, text); 
		text.LoadString(hDLL, IDS_CHT);
		subMenu->AppendMenu(MF_STRING, ID_LANGUAGE_CHINESETRADITIONAL, text); 
		text.LoadString(hDLL, IDS_CHS);
		subMenu->AppendMenu(MF_STRING, ID_LANGUAGE_CHINESESIMPLE, text); 

		//append submenu to menu
		text.LoadString(hDLL, IDS_LANGUAGE);
		menu->AppendMenu(MF_POPUP|MF_STRING, (UINT)subMenu->m_hMenu, text); 
		SetMenu(menu);

		SwitchLang(hDLL);
    }
	else
	{
		m_iLanguage = 0;
		hDLL= NULL;
	}

	if(menu)
	{
		menu->DestroyMenu();
		delete menu;
	}

	if(subMenu)
	{
		subMenu->DestroyMenu();
		delete subMenu;
	}
}

void CNuBridgeDlg::OnBnClickedRadioD1()
{
	// TODO: Add your control notification handler code here
	NuUsb.NU_SetDeviceLED(0);
	NuUsb.m_ActiveDevice = 0;
}

void CNuBridgeDlg::OnBnClickedRadioD2()
{
	// TODO: Add your control notification handler code here
	NuUsb.NU_SetDeviceLED(1);
	NuUsb.m_ActiveDevice = 1;
}

void CNuBridgeDlg::OnBnClickedRadioD3()
{
	// TODO: Add your control notification handler code here
	NuUsb.NU_SetDeviceLED(2);
	NuUsb.m_ActiveDevice = 2;
}

void CNuBridgeDlg::OnBnClickedRadioD4()
{
	// TODO: Add your control notification handler code here
	NuUsb.NU_SetDeviceLED(3);
	NuUsb.m_ActiveDevice = 3;
}

void CNuBridgeDlg::OnBnClickedClear()
{
	// TODO: Add your control notification handler code here
	m_MAIN_LIST.DeleteAllItems();
	nItem = 0;
	IDNum = 0;

}
/**
  * @brief  Nu-Bridge removal Message(OnMyDeviceChange).
  *         Get Nu-Bridge removal message           
  *             
  * @param  None.
  * @retval None.
  */
LRESULT CNuBridgeDlg::OnMyDeviceChange(WPARAM wParam, LPARAM lParam)
{
	if (DBT_DEVICEREMOVECOMPLETE == wParam) 
	{
		PDEV_BROADCAST_HDR pHdr = (PDEV_BROADCAST_HDR)lParam;
		PDEV_BROADCAST_DEVICEINTERFACE pDevInf;
		switch( pHdr->dbch_devicetype )
		{
			case DBT_DEVTYP_DEVICEINTERFACE:
				pDevInf = (PDEV_BROADCAST_DEVICEINTERFACE)pHdr;
				if(pDevInf->dbcc_classguid == GUID_DEVINTERFACE_LIST[0])
				{
					
					if(NuUsb.WinUsbNumber)
					{
						CString szDevId = pDevInf->dbcc_name;
						szDevId.MakeUpper();
						int devId = NuUsb.NU_CheckDevice(szDevId);

						DisableDevice(devId);					
					}
				}
				break;

			default:
				break;
			
		}
	}
	
	return 0;
}
/**
  * @brief  Detect Nu-Bridge removal function.
  *         UI will do something in this case.           
  *             
  * @param  None.
  * @retval None.
  */
void CNuBridgeDlg::DisableDevice(int devId)
{
	BOOL bFindOtherDev;  
	CString strerr;

	if(devId < 0)
		return;
	
	GetDlgItem(IDC_BUTTON_CONNECT)->EnableWindow(TRUE);
	m_tabCtrl.m_bLock = FALSE;
	m_tabCtrl.FunModuleDlg();

	bFindOtherDev = 0;
    	
	RadioValid[devId] = 1;
	
	if(devId == NuUsb.m_ActiveDevice)
	{		
		strerr.Format(_T("Removal active device %d"),(devId + 1));
		MessageBox(strerr);

		LockScreen(0 , 0);

		m_RADIO_Device[devId].SetCheck(FALSE);
		m_RADIO_Device[devId].EnableWindow(FALSE);

		// Find Next Active Device
		for(int i = 0; i < NuUsb.WinUsbNumber; i++)
		{
			if((i != NuUsb.m_ActiveDevice) && (RadioValid[i] == 0))
			{
				bFindOtherDev = 1;
				NuUsb.m_ActiveDevice = i;
				m_RADIO_Device[NuUsb.m_ActiveDevice].SetCheck(TRUE);
				SwitchUsbDevice(); 
	
				break;
			}
		}		
	}
	else   // removal deactive device
	{
		strerr.Format(_T("Removal Deactive device %d"),(devId + 1));
		MessageBox(strerr);
		return;
	}
	

	// Can't find another Nu-Bridge
	if(bFindOtherDev == 0)
	{
		
		EndUsbMonitor();
		bActiveUSB = 0;

		GetDlgItem(IDC_BTN_USB_ACTIVE)->ShowWindow(FALSE);		

		for(int i = 0; i < NuUsb.WinUsbNumber; i++)
		{
			m_RADIO_Device[i].ShowWindow(FALSE);		
		}
		NuUsb.WinUsbNumber = 0;
		MessageBox(_T("Nu-Bridge Not Found"));
	}

	OnBnClickedClear();  
}

BOOL CNuBridgeDlg::SetTipText(UINT id, NMHDR *pTTTStruct, LRESULT *pResult)
{
	TOOLTIPTEXT *pTTT = (TOOLTIPTEXT *)pTTTStruct;          
	UINT nID = pTTTStruct->idFrom;
	
	if (pTTT->uFlags & TTF_IDISHWND)  
	{
		nID = ::GetDlgCtrlID((HWND)nID);
		if (NULL == nID)
			return FALSE;

		switch(nID)
		{
			case IDC_BUTTON_CONNECT:				
			{
				if(hDLL == NULL)
					pTTT->lpszText = _T("Connect NuBridge.");
				else
				{
					ToolTipStr.LoadString(hDLL, IDS_TOOLTIP_CONNECT);
					pTTT->lpszText = (LPWSTR)((LPCTSTR)ToolTipStr);
				}
			}
			break;  
				
			case IDC_BTN_USB_ACTIVE:				
			{
				if(hDLL == NULL)
					pTTT->lpszText = _T("Selet active NuBride.");
				else
				{
					ToolTipStr.LoadString(hDLL, IDS_TOOLTIP_ACTIVE);
					pTTT->lpszText = (LPWSTR)((LPCTSTR)ToolTipStr);
				}
			}
			break;
			
			case IDC_CLEAR:			
			{
				if(hDLL == NULL)
					pTTT->lpszText = _T("Clear data.");
				else
				{
					ToolTipStr.LoadString(hDLL, IDS_TOOLTIP_CLEAR);
					pTTT->lpszText = (LPWSTR)((LPCTSTR)ToolTipStr);
				}
			}
			break;

			default:
				break;
		}
		return TRUE;
	}  
	return FALSE;    
}

LRESULT CNuBridgeDlg::LockListCtrl(WPARAM wParam, LPARAM lParam)
{
	BOOL bLock = (BOOL)(wParam);
	
	// Lock
	if(bLock)
	{
		GetDlgItem(IDC_MAIN_LIST)->EnableWindow(FALSE);	
	}
	else
	{
		GetDlgItem(IDC_MAIN_LIST)->EnableWindow(TRUE);	
	}

	return bLock;
}