#include "stdafx.h"
#include "Resource.h"
#include "NuTabCtrl.h"

// Parent
#include "NuBridgeDlg.h"

// Children
#include "SPI0_Master_Dlg.h"
#include "SPI_Monitor_Dlg.h"
#include "I2C0_Master_Dlg.h"
#include "GPIO_Monitor_Dlg.h"
#include "PWM_Capture_Dlg.h"
#include "PWM_Output_Dlg.h"
#include "ADC_Dlg.h"
/////////////////////////////////////////////////////////////////////////////
// CNuTabCtrl

CNuTabCtrl::CNuTabCtrl()
{
	::ZeroMemory(&m_tabPages, sizeof(m_tabPages));

	m_tabPages[0]=new SPI0_Master_Dlg;
	m_tabPages[1]=new SPI_Monitor_Dlg;
	m_tabPages[2]=new I2C0_Master_Dlg;
	m_tabPages[3]=new GPIO_Monitor_Dlg;
	m_tabPages[4]=new PWM_Capture_Dlg;
	m_tabPages[5]=new PWM_Output_Dlg;
	m_tabPages[6]=new ADC_Dlg;
	m_nNumberOfPages=7;
	m_bLock=0;
}

CNuTabCtrl::~CNuTabCtrl()
{
	for(int nCount=0; nCount < m_nNumberOfPages; nCount++){
		m_tabPages[nCount]->DestroyWindow();
		delete m_tabPages[nCount];
	}
}

void CNuTabCtrl::Init()
{
	m_tabCurrent=0;

	InsertItem(0, _T("SPI Master"));
	InsertItem(1, _T("SPI Monitor"));
	InsertItem(2, _T("I2C Master"));
	InsertItem(3, _T("I2C Monitor"));
	InsertItem(4, _T("PWM Capture"));
	InsertItem(5, _T("PWM Output"));
	InsertItem(6, _T("ADC"));

	m_tabPages[0]->Create(IDD_SPI0_MASTER_DIALOG  , this);
	m_tabPages[1]->Create(IDD_SPI_MONITOR_DIALOG  , this);
	m_tabPages[2]->Create(IDD_I2C0_MASTER_DIALOG  , this);
	m_tabPages[3]->Create(IDD_GPIO_MONITOR_DIALOG , this);
	m_tabPages[4]->Create(IDD_PWM_CAPTURE_DIALOG  , this);
	m_tabPages[5]->Create(IDD_PWM_OUTPUT_DIALOG , this);
	m_tabPages[6]->Create(IDD_ADC_DIALOG , this);

	m_tabPages[0]->ShowWindow(SW_SHOW);
	m_tabPages[1]->ShowWindow(SW_HIDE);
	m_tabPages[2]->ShowWindow(SW_HIDE);
	m_tabPages[3]->ShowWindow(SW_HIDE);
	m_tabPages[4]->ShowWindow(SW_HIDE);
	m_tabPages[5]->ShowWindow(SW_HIDE);
	m_tabPages[6]->ShowWindow(SW_HIDE);
	
	SetRectangle();
}

void CNuTabCtrl::SetRectangle()
{
	CRect tabRect, itemRect;
	int nX, nY, nXc, nYc;

	GetClientRect(&tabRect);
	GetItemRect(0, &itemRect);

	nX = itemRect.left;
	nY = itemRect.bottom+1;
	nXc = tabRect.right-itemRect.left-3;
	nYc = tabRect.bottom-nY-2;

	m_tabPages[0]->SetWindowPos(&wndTop, nX, nY, nXc, nYc, SWP_SHOWWINDOW);
	for(int nCount=1; nCount < m_nNumberOfPages; nCount++){
		m_tabPages[nCount]->SetWindowPos(&wndTop, nX, nY, nXc, nYc, SWP_HIDEWINDOW);
	}
}

void CNuTabCtrl::FunModuleDlg()
{
	((SPI0_Master_Dlg*)m_tabPages[0])->InitDialogWnd(true);
	((SPI_Monitor_Dlg*)m_tabPages[1])->InitDialogWnd(true);
	((I2C0_Master_Dlg*)m_tabPages[2])->InitDialogWnd(true);
	((GPIO_Monitor_Dlg*)m_tabPages[3])->InitDialogWnd(true);
	((PWM_Capture_Dlg*)m_tabPages[4])->InitDialogWnd(true);
	((PWM_Output_Dlg*)m_tabPages[5])->InitDialogWnd(true);
	((ADC_Dlg*)m_tabPages[6])->InitDialogWnd(true);
}

void CNuTabCtrl::SwitchLang(HINSTANCE LangDll, int LanguageIdx)	
{
	TabTitlesLanguage(LangDll, LanguageIdx);
	((SPI0_Master_Dlg*)m_tabPages[0])->SPI0SwitchLang(LangDll, LanguageIdx);
	((SPI_Monitor_Dlg*)m_tabPages[1])->SPIMSwitchLang(LangDll, LanguageIdx);
	((I2C0_Master_Dlg*)m_tabPages[2])->I2C0SwitchLang(LangDll, LanguageIdx);
	((GPIO_Monitor_Dlg*)m_tabPages[3])->I2CMSwitchLang(LangDll, LanguageIdx);
	((PWM_Capture_Dlg*)m_tabPages[4])->PWMCSwitchLang(LangDll, LanguageIdx);
	((PWM_Output_Dlg*)m_tabPages[5])->PWMOSwitchLang(LangDll, LanguageIdx);
	((ADC_Dlg*)m_tabPages[6])->ADCSwitchLang(LangDll, LanguageIdx);
	
	m_tabPages[m_tabCurrent]->RedrawWindow();
}

void CNuTabCtrl::TabTitlesLanguage(HINSTANCE LangDll, int LanguageIdx)	
{

	TCITEM tabItem;
	tabItem.mask = TCIF_TEXT;

	CString text;

	// Set the titles for each tab	
	if(LangDll && (LanguageIdx > 0))
	{
		text = LoadStringLang(LangDll, IDS_SPI0MASTER);
		tabItem.pszText = text.GetBuffer(0);
		SetItem(0, &tabItem);
		text.ReleaseBuffer();

		text = LoadStringLang(LangDll, IDS_SPIMONITOR);
		tabItem.pszText = text.GetBuffer(0);
		SetItem(1, &tabItem);
		text.ReleaseBuffer();

		text = LoadStringLang(LangDll, IDS_I2CMASTER);
		tabItem.pszText = text.GetBuffer(0);
		SetItem(2, &tabItem);
		text.ReleaseBuffer();

		text = LoadStringLang(LangDll, IDS_I2CMONITOR);
		tabItem.pszText = text.GetBuffer(0);
		SetItem(3, &tabItem);
		text.ReleaseBuffer();

		text = LoadStringLang(LangDll, IDS_PWM_CAPTURE);
		tabItem.pszText = text.GetBuffer(0);
		SetItem(4, &tabItem);
		text.ReleaseBuffer();

		text = LoadStringLang(LangDll, IDS_PWM_OUTPUT);
		tabItem.pszText = text.GetBuffer(0);
		SetItem(5, &tabItem);
		text.ReleaseBuffer();

		//text = LoadStringLang(LangDll, IDS_ADC_OUTPUT);
		//tabItem.pszText = text.GetBuffer(0);
		//SetItem(6, &tabItem);
		//text.ReleaseBuffer();

	}
	else
	{
		text = _T("SPI Master");
		tabItem.pszText = text.GetBuffer(0);
		SetItem(0, &tabItem);
		text.ReleaseBuffer();

		text = _T("SPI Monitor");
		tabItem.pszText = text.GetBuffer(0);
		SetItem(1, &tabItem);
		text.ReleaseBuffer();

		text = _T("I2C Master");
		tabItem.pszText = text.GetBuffer(0);
		SetItem(2, &tabItem);
		text.ReleaseBuffer();

		text = _T("I2C Monitor");
		tabItem.pszText = text.GetBuffer(0);
		SetItem(3, &tabItem);
		text.ReleaseBuffer();

		text = _T("PWM Capture");
		tabItem.pszText = text.GetBuffer(0);
		SetItem(4, &tabItem);
		text.ReleaseBuffer();

		text = _T("PWM Output");
		tabItem.pszText = text.GetBuffer(0);
		SetItem(5, &tabItem);
		text.ReleaseBuffer();

		text = _T("ADC");
		tabItem.pszText = text.GetBuffer(0);
		SetItem(6, &tabItem);
		text.ReleaseBuffer();
	}
}

CString CNuTabCtrl::LoadStringLang(HINSTANCE LangDll, UINT uID)
{
	
	CString text;

	if(LangDll)
	{
		text.LoadString(LangDll, uID);
	}
	else
		text.LoadString(uID);

	return text;
	
}

BEGIN_MESSAGE_MAP(CNuTabCtrl, CTabCtrl)
	//{{AFX_MSG_MAP(CNuTabCtrl)
	ON_WM_LBUTTONDOWN()
	ON_MESSAGE(WM_LOCK_SCREEN, &CNuTabCtrl::LockTabCtrl)

	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CNuTabCtrl message handlers

void CNuTabCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CTabCtrl::OnLButtonDown(nFlags, point);

	if(m_bLock)
	{
		SetCurFocus(m_tabCurrent);
		return;
	}
	
	if(m_tabCurrent != GetCurFocus()){
		m_tabPages[m_tabCurrent]->ShowWindow(SW_HIDE);
		m_tabCurrent = GetCurFocus();
		m_tabPages[m_tabCurrent]->ShowWindow(SW_SHOW);
		m_tabPages[m_tabCurrent]->SetFocus();
		
		GetCurrentModule();
		GetParent()->SendMessage( WM_TAB_PAGE_CLICKED, (WPARAM)m_tabCurrent );
	}
}

LRESULT CNuTabCtrl::LockTabCtrl(WPARAM wParam, LPARAM lParam)
{
	m_bLock = BOOL(wParam);
	GetParent()->SendMessage(WM_LOCK_SCREEN, wParam);

	// Lock List operation in PWM module
	if(5 == m_tabCurrent)
		GetParent()->SendMessage(WM_LOCK_LISTCTRL, wParam);

	UCHAR moduleId = GetCurrentModule();

	if(m_bLock)
		NuUsb.NU_SetBridgeMode(moduleId);
	else
		NuUsb.NU_SetBridgeStop(moduleId);


	return 1;
}

UCHAR CNuTabCtrl::GetCurrentModule()
{
	CNuBridgeDlg* pDlg = (CNuBridgeDlg*)GetParent();

	UCHAR moduleId;
	
	int Btn = 0;	// Module Button	--> 0: SPI, 1: I2C, 2: PWM, 3: ADC
	int Bmp = 0;	// Pin Bitmap		--> 0: SPI0, 1: SPI2, 2: I2C1, 3: I2C0, 4: PWM Capture, 4: PWM Output, 5: ADC
	
	
	switch(m_tabCurrent)
	{
		case 0:
			moduleId = MODE_SPI0_MASTER;
			if(((SPI0_Master_Dlg*)m_tabPages[m_tabCurrent])->m_iSpiID == 0)
			{
				moduleId = MODE_SPI0_MASTER;
				Btn = 0;	// SPI Button
				Bmp = 0;	// SPI0 Bitmap
				
			}
			else
			{
				moduleId = MODE_SPI2_MASTER;
				Btn = 0;	// SPI Button
				Bmp = 1;	// SPI2 Bitmap
			}
			break;

		case 1:
			moduleId = MODE_SPI_MONITOR;
			Btn = 0;	// SPI Button
			Bmp = 0;	// SPI0 Bitmap
			break;

		case 2:
			moduleId = MODE_I2C0_MASTER;
			Btn = 1;	// I2C Button
			Bmp = 2;	// I2C1 Bitmap
			break;

		case 3:
			moduleId = MODE_I2C_MONITOR;
			Btn = 1;	// I2C Button
			Bmp = 2;	// I2C1 Bitmap
			break;

		case 4:
			moduleId = MODE_PWM;
			Btn = 2;	// PWM Button
			Bmp = 3;	// PWM Capture Bitmap
			break;

		case 5:
			moduleId = MODE_PWM;
			Btn = 2;	// PWM Button
			Bmp = 4;	// PWM Output Bitmap
			break;

		case 6:
			moduleId = MODE_ADC;
			Btn = 3;	// ADC Button
			Bmp = 5;	// ADC Bitmap
			break;
	}

	pDlg->m_MainFrameWnd.SwitchContent(Btn, Bmp);
	
	
	return moduleId;
}

