/**************************************************************************//**
 * @file     NuWidget.cpp
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 13/12/03 11:27a $
 * @brief    Worker thread, String parsing  and String format module
 *
 * @note
 * Copyright (C) 2013 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include "StdAfx.h"
#include "NuBridgeDlg.h"
#include "NuWidget.h"
#include "PWM_Capture_Dlg.h"
extern int m_iLanguage;

UCHAR InitThreadData();
UCHAR ShowI2cMonitor2();


// USB buffer for Thread Read and Write
#define BLOCK_CNT	1024
#define BOUND		(BLOCK_CNT - 1)
BOOL g_bThreadContinue = FALSE;
volatile UCHAR UsbBufSize[BLOCK_CNT];
UCHAR UsbBuf[BLOCK_CNT][64];
volatile int UsbBufTail = 0;
volatile int UsbBufHead = 0;

CColorListCtrl *pListCtl = NULL;

const char HexTable[16] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
const TCHAR THexTable[16] = {_T('0'), _T('1'), _T('2'), _T('3'), _T('4'), _T('5'), _T('6'), _T('7'), _T('8'), _T('9'), _T('A'), _T('B'), _T('C'), _T('D'), _T('E'), _T('F')};

CString strTextMISO;
int strLengthSPI0_MISO = 0;

CString strTextHex;
int strLengthHex = 0;

int IDNum = 0;
int nItem = 0; 


UCHAR SPI0ReadBuf[1024] = {0};
extern UCHAR ScriptDataI2C[1024];

BOOL g_bSPIMException = FALSE;
BOOL g_bPWMException = FALSE;

extern int edge_type0;
extern int edge_type1;
enum EDGETYPE {FALL, RISE, BOTH};
enum STATE {STATE_START, STATE_FALLING0, STATE_RISING0, STATE_FALLING1, STATE_RISING1, STATE_READY0, STATE_READY1};
int Edge0, FallWidth0, RiseWidth0, Edge1, FallWidth1, RiseWidth1; 
float duty0, duty1;

extern float InputVol;

extern UCHAR g_u8DataFilterR, g_u8DataFilterW, g_u8DataFilterACK, g_u8DataFilterNACK;
extern UCHAR g_u8DeviceFilterAll, g_u8DeviceFilterDevice;
extern UCHAR g_u8DeviceFilterAdd;


std::string byte2str(UCHAR* bytes, int size)
{
	std::string str;
	for(int i = 0; i < size; i++)
	{
		const char ch = bytes[i];
		str.append(&HexTable[(ch & 0xF0) >> 4], 1);
		str.append(&HexTable[(ch & 0x0F)], 1);
		str.append(" ");
	}
	return str;
}

std::wstring byte2wstr(UCHAR* bytes, int size)
{
	std::wstring str;
	for(int i = 0; i < size; i++)
	{
		const char ch = bytes[i];
		str.append(&THexTable[(ch & 0xF0) >> 4], 1);
		str.append(&THexTable[(ch & 0x0F)], 1);
		str.append(_T(" "));
	}
	return str;
}

CString ShowErrMsg(UCHAR Status)
{
    CString str;
	HMODULE hListDLL;

	hListDLL = pListCtl->m_hLangDLL;
	
	switch(Status)
	{
		case 0x00:
		{					
			if(hListDLL == NULL)
				str = _T("Bus error.");	
			else
				str.LoadString(hListDLL, IDS_ERRMSG1);
		}
		break;
		
		case 0x20:	
			if(hListDLL == NULL)
				str = _T("SLA+W sent, NACK received.");
			else
				str.LoadString(hListDLL, IDS_ERRMSG2);
		break;
		
		case 0x30:
			if(hListDLL == NULL)
				str = _T("Data sent, NACK received.");
			else
				str.LoadString(hListDLL, IDS_ERRMSG3);	
		break;
		
		case 0x38:		
			if(hListDLL == NULL)
				str = _T("Arbitration Lost.");
			else
				str.LoadString(hListDLL, IDS_ERRMSG5);
		break;
		
		case 0x48:
			if(hListDLL == NULL)
				str = _T("SLA+R sent, NACK received.");
			else
				str.LoadString(hListDLL, IDS_ERRMSG4);
		break;
		
		default:		
			if(hListDLL == NULL)
				str = _T("Time out.");
			else
				str.LoadString(hListDLL, IDS_ERRMSG6);
		break;
	}
	
	return str;
}

UINT UsbReadThread(LPVOID pVar)
{
	int ret = 0;

	int DeviceId = NuUsb.m_ActiveDevice;
	NuUsb.m_WorkDevice = DeviceId;	

	InitThreadData();
	g_bThreadContinue = TRUE;
	UsbBufTail = 0;
	memset((void *)UsbBufSize, 0, BLOCK_CNT);

	
	
	
	
	while(g_bThreadContinue)
	{
		if (UsbBufSize[UsbBufTail])
			continue;

		ret = NuUsb.NU_ReadPipe(DeviceId, (UCHAR *)(&UsbBuf[UsbBufTail][0]), 64);		

		if(ret)
		{
			UsbBufSize[UsbBufTail] = ret;
			UsbBufTail++;
			UsbBufTail &= BOUND;
		}
	};

	UsbBufTail = 0;

	return 0x12;
}


CString strTextI2C;


// I2C status
UCHAR ch = 0;
UCHAR bitCnt = 0;
UCHAR Ack = 0;
UCHAR Addr = 0;


UCHAR ShowI2cMonitor(int PacketSize)
{
	int Offset = UsbBuf[UsbBufHead][1] + 4;	
	UCHAR *Watch = &(UsbBuf[UsbBufHead][Offset]);;


	
	for(int i = Offset; i < PacketSize; i++)
	{
		switch (UsbBuf[UsbBufHead][i])
		{
			case 0:// data : ack nack read and write
			case 1:// data : ack nack read and write
				ch = ch * 2 + UsbBuf[UsbBufHead][i];
				if(Ack)
				{
					if(Ack == 2)
					{
						if(ch)
						{
							strTextI2C.AppendChar(_T('"'));
							strTextI2C.AppendChar(_T('R'));
							strTextI2C.AppendChar(_T(' '));
						}
						else
						{
							strTextI2C.AppendChar(_T('"'));
							strTextI2C.AppendChar(_T('W'));
							strTextI2C.AppendChar(_T(' '));
						}								
					}
					else
					{
					/*
						if(ch)
						{
							strTextI2C.AppendChar(_T(' '));
							strTextI2C.AppendChar(_T('N'));
							strTextI2C.AppendChar(_T(' '));
						}
						else
						{
							strTextI2C.AppendChar(_T(' '));
							strTextI2C.AppendChar(_T('A'));
							strTextI2C.AppendChar(_T(' '));
						}
					*/								
					}
										
					ch = 0;
					bitCnt = 0;
					Ack--;// = 0;
				}
				else{
					bitCnt++;
					if(bitCnt == 8)
					{
						strTextI2C.AppendChar(THexTable[(ch & 0xF0) >> 4]);
						strTextI2C.AppendChar(THexTable[(ch & 0x0F)]);						
						strTextI2C.AppendChar(_T(' '));
					
						ch = 0;
						bitCnt = 0;
						Ack = 1 + Addr;
						Addr = 0;
					}
				}
				break;
			
			case 2:  //start
				if(bitCnt)
				{

					strTextI2C.AppendChar(THexTable[(ch & 0xF0) >> 4]);
					strTextI2C.AppendChar(THexTable[(ch & 0x0F)]);						
					strTextI2C.AppendChar(_T(' '));
					
					// Change Line
					pListCtl->SetItemColor(nItem, RGB(0,0,0));
					pListCtl->InsertItem(nItem, _T(""));
					pListCtl->SetItemText(nItem++, 1, strTextI2C);

					strTextI2C = _T("");
				
					ch = 0;
				}
				
				strTextI2C.AppendChar(_T('['));
				strTextI2C.AppendChar(_T('S'));
				strTextI2C.AppendChar(_T(']'));
				strTextI2C.AppendChar(_T(' '));
				

				Addr = 1;

				bitCnt = 1;
				ch = 0;

				break;
				
			case 6: //stop
//						if(bitCnt)

				strTextI2C.AppendChar(_T('['));
				strTextI2C.AppendChar(_T('P'));
				strTextI2C.AppendChar(_T(']'));
				strTextI2C.AppendChar(_T(' '));
				
				// Change Line
				pListCtl->SetItemColor(nItem, RGB(0,0,0));
				pListCtl->InsertItem(nItem, _T(""));
				pListCtl->SetItemText(nItem++, 1, strTextI2C);

				strTextI2C = _T("");
				
				ch = 0;
				bitCnt = 0;
				break;
		}
	}
	return 0;
}

UCHAR g_bI2cHeader = 0;
UCHAR g_I2cStatus = 0;
UCHAR g_I2cErrCmd = 0;
UCHAR g_I2cErrCnt = 0;
UCHAR g_I2cActiveCmd = 0;
USHORT g_I2cValidCnt = 0;

USHORT g_I2cScriptPointer = 1;
UCHAR g_I2cCmdLength = 0;
UCHAR g_I2cRdDataCnt = 0;


bool ErrFlag = 0, bI2cIDPrint = 0;
CString strI2cMaster;
int LineCnt = 0, TempLineCnt = 0;
UCHAR ShowI2c0Master(int PacketSize)
{
	int Offset = UsbBuf[UsbBufHead][1] + 4;	
	UCHAR *Watch = &(UsbBuf[UsbBufHead][Offset]);
	int resData = PacketSize - Offset;
	
	UCHAR CmdLength = 0;
	UCHAR cmd; // = ScriptDataI2C[g_I2cScriptPointer++];
	UCHAR addr; // = ScriptDataI2C[g_I2cScriptPointer++];
	
	CString StrStatus;
	while(resData)
	{
		if(ScriptDataI2C[0] == g_I2cActiveCmd)
			return 0;
	
		switch(g_bI2cHeader)
		{
			// case 0~4: Get Command Error information from Data Packet
			// These information could be stored in the end of one USB packet and 
			// also in the begining of the other USB packet.
			// So, I use 5 cases istead of 1 to get these 5 Bytes information.
			
			case 0:		// Init Status
				g_I2cStatus = *Watch++;
				g_bI2cHeader++;
				resData--;
			break;
			
			case 1:
				g_I2cErrCmd = *Watch++;
				g_bI2cHeader++;
				resData--;
			break;
				
			case 2:
				g_I2cErrCnt = *Watch++;
				g_bI2cHeader++;
				resData--;
			break;
			
			case 3:
				g_I2cValidCnt = *Watch++;
				g_bI2cHeader++;
				resData--;
			break;
				
			case 4:
				g_I2cValidCnt += ((*Watch) * 256);
				Watch++;
				g_bI2cHeader++;
				resData--;
				
				
			// case 5~6: Get Command Data from Data Packet
			// if Data is enough, just display the command in List Control.
			// else Jump to case 6 to wait for next Data Packet
			
			case 5:  // Parsing USB Data and Script Data
			
				while(ScriptDataI2C[0] != g_I2cActiveCmd && g_bI2cHeader)	// g_bI2cHeader could be 0
				{
					ErrFlag = FALSE;
					
					// Get First Command From user input script data
					cmd = ScriptDataI2C[g_I2cScriptPointer++];
					addr = ScriptDataI2C[g_I2cScriptPointer++];
					g_I2cRdDataCnt = ScriptDataI2C[g_I2cScriptPointer++];

					// Error Message
					if(g_I2cStatus != 1 && g_I2cErrCmd == g_I2cActiveCmd)
					{
						ErrFlag = TRUE;
					}
					
					if('W' == cmd)
					{
						// Success Message
						if(ErrFlag)
						{
							CString ErrMsg;
						
							pListCtl->SetItemColor(nItem, RGB(255,0,0));
							ErrMsg.Format(_T("NG   #%d"), g_I2cActiveCmd);
							pListCtl->InsertItem(nItem, ErrMsg);			
							ErrMsg = ShowErrMsg(g_I2cStatus);
							pListCtl->SetItemText(nItem++, 1, ErrMsg);														
							g_I2cScriptPointer += g_I2cRdDataCnt;
							g_bI2cHeader = 0;
						}
						else
						{
							LineCnt = g_I2cRdDataCnt;

							if(g_I2cRdDataCnt > 64)
							{							
								strI2cMaster.Format(_T("[S] %2X \"W "), addr);
								StrStatus.Format(_T("Pass #%d"), g_I2cActiveCmd); 
								while(LineCnt > 0)
								{		
									if(LineCnt > 64)
									{
										strI2cMaster += byte2wstr(&ScriptDataI2C[g_I2cScriptPointer], 64).c_str();																				
										TempLineCnt = 64;
									}
									else
									{										
										strI2cMaster += byte2wstr(&ScriptDataI2C[g_I2cScriptPointer], TempLineCnt).c_str();											
										strI2cMaster += _T("[P]");
									}
																		
									g_I2cScriptPointer += TempLineCnt;

									LineCnt -= 64;
									TempLineCnt = LineCnt;
									
									pListCtl->SetItemColor(nItem, RGB(0,0,0));									
									pListCtl->InsertItem(nItem, StrStatus);
									StrStatus = _T("");
									pListCtl->SetItemText(nItem++, 1, strI2cMaster);
									strI2cMaster = _T("          ");
								}								
							}	
							else
							{
								strI2cMaster.Format(_T("[S] %2X \"W "), addr);
								strI2cMaster += byte2wstr(&ScriptDataI2C[g_I2cScriptPointer], g_I2cRdDataCnt).c_str();
								strI2cMaster += _T("[P]");
								
								pListCtl->SetItemColor(nItem, RGB(0,0,0));
								StrStatus.Format(_T("Pass #%d"), g_I2cActiveCmd);
								pListCtl->InsertItem(nItem, StrStatus);
								StrStatus = _T("");
								pListCtl->SetItemText(nItem++, 1, strI2cMaster);		
								g_I2cScriptPointer += g_I2cRdDataCnt;															
							}
						}										
						g_I2cActiveCmd ++;					
					}
					else if('R' == cmd)	// Get Read data from USB return packet
					{
						bI2cIDPrint = 0;
						
						strI2cMaster.Format(_T("[S] %2X \"R "), addr);	
						StrStatus.Format(_T("Pass #%d"), g_I2cActiveCmd);					
						// Error case in read command
						if(ErrFlag)
						{
							CString ErrMsg;											
							pListCtl->SetItemColor(nItem, RGB(255,0,0));							
							ErrMsg.Format(_T("NG   #%d"), g_I2cActiveCmd);
							pListCtl->InsertItem(nItem, ErrMsg);
							ErrMsg = ShowErrMsg(g_I2cStatus);
							pListCtl->SetItemText(nItem++, 1,  ErrMsg);	
						
							g_I2cRdDataCnt = g_I2cValidCnt;						
							resData -= g_I2cRdDataCnt;
							g_I2cValidCnt -= g_I2cRdDataCnt;
							Watch += g_I2cRdDataCnt;

							g_bI2cHeader = 0;
						}						
						else
						{
							LineCnt = g_I2cRdDataCnt;						
														
							if(g_I2cRdDataCnt > resData)
							{	
								strI2cMaster += byte2wstr(Watch, resData).c_str();
								g_I2cRdDataCnt -= resData;								
								g_I2cValidCnt -= resData;
								LineCnt -= resData;
								g_bI2cHeader = 6;	// Not Finished														
								TempLineCnt = resData;
								return 0;
							}
							else
							{
								LineCnt = g_I2cRdDataCnt;
								
								if(g_I2cRdDataCnt > 64)
								{							
									while(LineCnt > 0)
									{
										
										if(LineCnt > 64)
										{
											strI2cMaster += byte2wstr(Watch, 64).c_str();
											
											TempLineCnt = 64;
										}
										else
										{											
											strI2cMaster += byte2wstr(Watch, TempLineCnt).c_str();
											strI2cMaster += _T("[P]");
										}
										
										Watch += TempLineCnt;
										resData -= TempLineCnt;
										g_I2cValidCnt -= TempLineCnt;
								
										g_I2cRdDataCnt -= TempLineCnt;
										LineCnt -= 64;
										TempLineCnt = LineCnt;
										
										pListCtl->SetItemColor(nItem, RGB(0,0,0));
										if(bI2cIDPrint == 0)										
										{
											pListCtl->InsertItem(nItem, StrStatus);
											bI2cIDPrint = 1;
										}
										else
											pListCtl->InsertItem(nItem,  _T(""));
										
										pListCtl->SetItemText(nItem++, 1, strI2cMaster);										
										strI2cMaster = _T("          ");
									}
								}
								else
								{								
									strI2cMaster += byte2wstr(Watch, g_I2cRdDataCnt).c_str();
									strI2cMaster += _T("[P]");
									resData -= g_I2cRdDataCnt;
									g_I2cValidCnt -= g_I2cRdDataCnt;
									Watch += g_I2cRdDataCnt;								
									bI2cIDPrint = 0;
									pListCtl->SetItemColor(nItem, RGB(0,0,0));
									if(bI2cIDPrint == 0)										
									{
										pListCtl->InsertItem(nItem, StrStatus);
										bI2cIDPrint = 1;
									}
									else
										pListCtl->InsertItem(nItem,  _T(""));
									pListCtl->SetItemText(nItem++, 1, strI2cMaster);
								}
							}							
						}						
						g_I2cActiveCmd ++;				
					}					
				}
			break;
				
			case 6:
				StrStatus.Format(_T("Pass #%d"), g_I2cActiveCmd);
				if(LineCnt > 64)   // data > 64
				{	
					LineCnt = g_I2cValidCnt;
					if(g_I2cRdDataCnt > resData)
					{
						
						TempLineCnt += resData; 
						if(TempLineCnt > 64)
						{
							strI2cMaster += byte2wstr(Watch, resData - (TempLineCnt - 64)).c_str();
							
							pListCtl->SetItemColor(nItem, RGB(0,0,0));								
							if(bI2cIDPrint == 0)										
							{
								pListCtl->InsertItem(nItem, StrStatus);
								bI2cIDPrint = 1;
							}
							else
								pListCtl->InsertItem(nItem,  _T(""));

							pListCtl->SetItemText(nItem++, 1, strI2cMaster);
							strI2cMaster = _T("          ");
							
							Watch += (resData - (TempLineCnt - 64));
							g_I2cValidCnt -= (resData - (TempLineCnt - 64));
							g_I2cRdDataCnt -= (resData - (TempLineCnt - 64));
							LineCnt -= (resData - (TempLineCnt - 64));//-= 64;
							TempLineCnt -= 64;
															
							if(TempLineCnt > 0)
							{								
								strI2cMaster += byte2wstr(Watch, TempLineCnt).c_str();										
								LineCnt -= TempLineCnt;
								g_I2cValidCnt -= TempLineCnt;
								g_I2cRdDataCnt -= TempLineCnt;
							}
							
						}
						else
						{							
							strI2cMaster += byte2wstr(Watch, resData).c_str();
							LineCnt -= resData;
							g_I2cRdDataCnt -= resData;
						}
						
						g_bI2cHeader = 6;
						
						return 0;	// Not Finished
					}
					else
					{
						TempLineCnt += resData; 
						if(TempLineCnt > 64)
						{
							strI2cMaster += byte2wstr(Watch, resData - (TempLineCnt - 64)).c_str();
							pListCtl->SetItemColor(nItem, RGB(0,0,0));							
							if(bI2cIDPrint == 0)										
							{
								pListCtl->InsertItem(nItem, StrStatus);
								bI2cIDPrint = 1;
							}
							else
								pListCtl->InsertItem(nItem,  _T(""));
								
							pListCtl->SetItemText(nItem++, 1, strI2cMaster);
							strI2cMaster = _T("          ");
							
							Watch += (resData - (TempLineCnt - 64));
							g_I2cValidCnt -= (resData - (TempLineCnt - 64));
							g_I2cRdDataCnt -= (resData - (TempLineCnt - 64));
							TempLineCnt -= 64;

							if(TempLineCnt > 0)
							{
								strI2cMaster += byte2wstr(Watch, TempLineCnt).c_str();										
								LineCnt -= TempLineCnt;
								g_I2cValidCnt -= TempLineCnt;
								g_I2cRdDataCnt -= TempLineCnt;
							}							
						}
						else
						{
							strI2cMaster += byte2wstr(Watch, resData).c_str();

							pListCtl->SetItemColor(nItem, RGB(0,0,0));
							if(bI2cIDPrint == 0)										
							{
								pListCtl->InsertItem(nItem, StrStatus);
								bI2cIDPrint = 1;
							}
							else
								pListCtl->InsertItem(nItem,  _T(""));
								
							pListCtl->SetItemText(nItem++, 1, strI2cMaster);
							strI2cMaster = _T("          ");

							g_I2cValidCnt -= resData;
							g_I2cRdDataCnt -= resData;
							LineCnt -= resData;
							
							if(g_I2cErrCmd == g_I2cActiveCmd)
								g_bI2cHeader = 0;

							g_I2cActiveCmd ++;				
							
							if(g_I2cRdDataCnt == 0 && g_I2cValidCnt >= 0)
							{
								g_bI2cHeader = 5;
							}
						}						
					}
				}
				else   // data < 64
				{
					int ProcLineCnt = 0;
						
					if(g_I2cRdDataCnt > resData)
					{
						ProcLineCnt = TempLineCnt;
						if(resData > g_I2cRdDataCnt)
						{						
							TempLineCnt += g_I2cRdDataCnt;
						}
						else
						{
							 TempLineCnt += resData;
						}

						if(TempLineCnt > 64)
						{
							strI2cMaster += byte2wstr(Watch, (64 - ProcLineCnt)).c_str();
							pListCtl->SetItemColor(nItem, RGB(0,0,0));
							
							if(bI2cIDPrint == 0)										
							{
								pListCtl->InsertItem(nItem, StrStatus);
								bI2cIDPrint = 1;
							}
							else
								pListCtl->InsertItem(nItem,  _T(""));

							pListCtl->SetItemText(nItem++, 1, strI2cMaster);
							strI2cMaster = _T("          ");

							Watch += (64 - ProcLineCnt);
							resData -= (64 - ProcLineCnt);
							g_I2cRdDataCnt -= (64 - ProcLineCnt);

							TempLineCnt -= 64;
							LineCnt -= (64 - ProcLineCnt);							

							if(TempLineCnt > 0 && g_I2cRdDataCnt > 0)
							{
								
								strI2cMaster += byte2wstr(Watch, TempLineCnt).c_str();
								strI2cMaster += _T("[P]");
								Watch += TempLineCnt;
								
								LineCnt -= TempLineCnt;
								TempLineCnt -= TempLineCnt;	

								pListCtl->SetItemColor(nItem, RGB(0,0,0));
								if(bI2cIDPrint == 0)										
								{
									pListCtl->InsertItem(nItem, StrStatus);
									bI2cIDPrint = 1;
								}
								else
									pListCtl->InsertItem(nItem,  _T(""));
								pListCtl->SetItemText(nItem++, 1, strI2cMaster);
								strI2cMaster = _T("          ");								
							}
							else
							{														
								strI2cMaster += byte2wstr(Watch, TempLineCnt).c_str();	
								Watch += TempLineCnt;
								
								TempLineCnt -= TempLineCnt;								
								LineCnt -= TempLineCnt;
							}							

							g_I2cRdDataCnt -= TempLineCnt;
							g_I2cValidCnt -= resData;

							if(g_I2cRdDataCnt == 0 && g_I2cValidCnt >= 0)
							{
								g_bI2cHeader = 5;
							}
						}
						else
						{							
							strI2cMaster += byte2wstr(Watch, resData).c_str();
							LineCnt -= resData;
							g_I2cValidCnt -= resData;
							g_I2cRdDataCnt -= resData;
						}						
						
						g_bI2cHeader = 6;
						
						return 0;	// Not Finished
					}
					else
					{	
						ProcLineCnt = TempLineCnt;

						if(resData > g_I2cRdDataCnt)
						{
							TempLineCnt += g_I2cRdDataCnt;
						}
						else
						{
							 TempLineCnt += resData;
						}
						
						if(TempLineCnt > 64)
						{
							strI2cMaster += byte2wstr(Watch, (64 - ProcLineCnt)).c_str();
							pListCtl->SetItemColor(nItem, RGB(0,0,0));

							if(bI2cIDPrint == 0)										
							{
								pListCtl->InsertItem(nItem, StrStatus);
								bI2cIDPrint = 1;
							}
							else
								pListCtl->InsertItem(nItem,  _T(""));							

							pListCtl->SetItemText(nItem++, 1, strI2cMaster);
							strI2cMaster = _T("          ");
							
							g_I2cValidCnt -= (64 - ProcLineCnt);
							Watch += (64 - ProcLineCnt);
							resData -= (64 - ProcLineCnt);
							g_I2cRdDataCnt -= (64 - ProcLineCnt);
							
							TempLineCnt -= 64;
							LineCnt -= (64 - ProcLineCnt);
							

							if(TempLineCnt > 0 && g_I2cRdDataCnt > 0)
							{								
								strI2cMaster += byte2wstr(Watch, TempLineCnt).c_str();
								strI2cMaster += _T("[P]");
								Watch += TempLineCnt;
								g_I2cRdDataCnt -= TempLineCnt;
								resData -= TempLineCnt;
								g_I2cValidCnt -= TempLineCnt;
								LineCnt -= TempLineCnt;
								TempLineCnt -= TempLineCnt;	

								pListCtl->SetItemColor(nItem, RGB(0,0,0));
								if(bI2cIDPrint == 0)										
								{
									pListCtl->InsertItem(nItem, StrStatus);
									bI2cIDPrint = 1;
								}
								else
									pListCtl->InsertItem(nItem,  _T(""));

								pListCtl->SetItemText(nItem++, 1, strI2cMaster);
								strI2cMaster = _T("          ");
							}
							else
							{											
								strI2cMaster += byte2wstr(Watch, TempLineCnt).c_str();	
								Watch += TempLineCnt;
								g_I2cRdDataCnt -= TempLineCnt;
								resData -= TempLineCnt;
								g_I2cValidCnt -= TempLineCnt;
								TempLineCnt -= TempLineCnt;								
								LineCnt -= TempLineCnt;
							}
							
						}						
						else
						{							
							strI2cMaster += byte2wstr(Watch, g_I2cRdDataCnt).c_str();
							strI2cMaster += _T("[P]");
							Watch += g_I2cRdDataCnt;
							resData -= g_I2cRdDataCnt;
							g_I2cRdDataCnt -= TempLineCnt;
							g_I2cValidCnt -= g_I2cRdDataCnt;
							pListCtl->SetItemColor(nItem, RGB(0,0,0));
							
							if(bI2cIDPrint == 0)										
							{
								pListCtl->InsertItem(nItem, StrStatus);
								bI2cIDPrint = 1;
							}
							else
								pListCtl->InsertItem(nItem,  _T(""));
								
							pListCtl->SetItemText(nItem++, 1, strI2cMaster);
							strI2cMaster = _T("          ");
						}
						if(g_I2cErrCmd == g_I2cActiveCmd)
							g_bI2cHeader = 0;
						
						g_I2cActiveCmd ++;
						
						// resData is not empty, but cmd length zero so here will go to next cmd to process.
						if(g_I2cRdDataCnt == 0 && g_I2cValidCnt >= 0)
						{
							g_bI2cHeader = 5;
						}						
					}
				}
				
			break;
		}	// switch(g_bI2cHeader)
	}	// while(resData)

	if(ScriptDataI2C[0] == g_I2cActiveCmd)
		return 0;
	
	// Exception case,
	// Error command without any Data available is processed at last
	
	if(g_I2cActiveCmd  && (g_bI2cHeader >= 5))  //&& (0 == g_I2cValidCnt)
	{
		g_bI2cHeader = 0;

		int CmdEndIdx = 0;


		if(1 == g_I2cStatus)
		{
			CmdEndIdx = ScriptDataI2C[0] - 1;
		}
		else
		{
			CmdEndIdx = g_I2cErrCmd;
		}
		
		while(g_I2cActiveCmd <= CmdEndIdx)
		{		
			// Get non processed error Command From user input script data
			cmd = ScriptDataI2C[g_I2cScriptPointer++];
			addr = ScriptDataI2C[g_I2cScriptPointer++];
			g_I2cRdDataCnt = ScriptDataI2C[g_I2cScriptPointer++];
			
			if(g_I2cStatus != 1 && g_I2cErrCmd == g_I2cActiveCmd)
			{				
				ErrFlag = TRUE;
			}
			if('W' == cmd)
			{
				if(ErrFlag)
				{
					CString ErrMsg;
									
					pListCtl->SetItemColor(nItem, RGB(255,0,0));
					ErrMsg.Format(_T("NG   #%d"), g_I2cActiveCmd);
					pListCtl->InsertItem(nItem, ErrMsg);
					ErrMsg = ShowErrMsg(g_I2cStatus);
					pListCtl->SetItemText(nItem++, 1, ErrMsg);

					g_I2cScriptPointer += g_I2cRdDataCnt;
					g_I2cActiveCmd ++;						
				}	
				else
				{
					// Success Message
					strI2cMaster.Format(_T("[S] %2X W\" "), addr);
					strI2cMaster += byte2wstr(&ScriptDataI2C[g_I2cScriptPointer], g_I2cRdDataCnt).c_str();
					strI2cMaster += _T("[P]");
					
					pListCtl->SetItemColor(nItem, RGB(0,0,0));
					StrStatus.Format(_T("Pass #%d"), g_I2cActiveCmd);
					pListCtl->InsertItem(nItem, StrStatus);
					pListCtl->SetItemText(nItem++, 1, strI2cMaster);

					g_I2cScriptPointer += g_I2cRdDataCnt;
					g_I2cActiveCmd ++;
				}
			}
			else if('R' == cmd)	// Get Read data from USB return packet
			{
				if(ErrFlag)
				{
					CString ErrMsg;
									
					pListCtl->SetItemColor(nItem, RGB(255,0,0));
					ErrMsg.Format(_T("NG   #%d"), g_I2cActiveCmd);					
					pListCtl->InsertItem(nItem, ErrMsg);					
					ErrMsg = ShowErrMsg(g_I2cStatus);
					pListCtl->SetItemText(nItem++, 1, ErrMsg);		

					g_I2cScriptPointer += g_I2cRdDataCnt;
					g_I2cActiveCmd ++;							
				}	
				else
				{
					strI2cMaster.Format(_T("[S] %2X R\" "), addr);
					
					g_I2cRdDataCnt = 0;
						
					strI2cMaster += byte2wstr(Watch, g_I2cRdDataCnt).c_str();
					strI2cMaster += _T("[P]");
					
					pListCtl->SetItemColor(nItem, RGB(0,0,0));
					StrStatus.Format(_T("Pass #%d"), g_I2cActiveCmd);
					pListCtl->InsertItem(nItem, StrStatus);
					pListCtl->SetItemText(nItem++, 1, strI2cMaster);
					
					g_I2cActiveCmd ++;
				}
			}			
		}
	}

	return 0;
}

CString strSPI0IDItem;
int SPI0_IDNum = 0;

int g_iScriptIdx = 0;
int g_iScriptStart = 0;

int g_iCmdLegth = 0;
int g_iCmdRepeat = 0;
int g_iDataLegth = 0;
int g_iDataCnt = 0;

int g_iSPIDataTmpRx = 0;
int g_iSPIDataTmpTx = 0;
int g_iSPIDataCnt = 0;

int g_iStrLength = 0;

CString strSPIRx;
CString strSPITx;
int strSPILength = 0;
BOOL bPrintIDFlag = FALSE;


UCHAR ShowSpi0Master(int PacketSize)
{
	int Offset = UsbBuf[UsbBufHead][1] + 4;	
	UCHAR *Watch = &(UsbBuf[UsbBufHead][Offset]);
	int resData = PacketSize - Offset;
	
	
	if(0 == g_iScriptIdx)
	{
		g_iScriptStart = 1;

		SPI0_IDNum = 0;
		
		g_iScriptIdx = 1;
		
		// Process Command Header		
		g_iScriptIdx++;	// Start 'S'
		g_iCmdRepeat = ScriptData[g_iScriptIdx++];
		g_iScriptIdx+= 2;	// Sikp Idle Time
		
		// Process Block Header
		g_iDataLegth = (ScriptData[g_iScriptIdx++] + 7) >> 3;
		g_iSPIDataCnt = ScriptData[g_iScriptIdx++];
		
		g_iDataCnt = 0;
		strSPIRx = _T("");
		strSPITx = _T("");
		strSPILength = 0;	
		
		bPrintIDFlag = TRUE;
	}
	
	bool BlockDone = true;
	
	for(int i = 0; i < resData; i++)
	{
		BlockDone = true;

		// Check Block Data Cnt
		if(g_iSPIDataCnt)
		{
			BlockDone = false;
		
			switch(g_iDataCnt)
			{
				case 0:
					g_iSPIDataTmpTx = ScriptData[g_iScriptIdx++];
					g_iSPIDataTmpRx = Watch[i];
					g_iDataCnt++;
					strSPILength++;
					break;
				case 1:
					g_iSPIDataTmpTx += (ScriptData[g_iScriptIdx++] << 8);
					g_iSPIDataTmpRx += (Watch[i] << 8);
					g_iDataCnt++;
					strSPILength++;
					break;
				case 2:
					g_iSPIDataTmpTx += (ScriptData[g_iScriptIdx++] << 16);
					g_iSPIDataTmpRx += (Watch[i] << 16);
					g_iDataCnt++;
					strSPILength++;
					break;
				case 3:
					g_iSPIDataTmpTx += (ScriptData[g_iScriptIdx++] << 24);
					g_iSPIDataTmpRx += (Watch[i] << 24);
					g_iDataCnt++;
					strSPILength++;
					break;
			}
				
			if(g_iDataLegth == g_iDataCnt)
			{
				
				BlockDone = true;
				g_iDataCnt = 0;
				g_iSPIDataCnt--;
				CString strTmpRx;
				CString strTmpTx;
				if(g_iDataLegth == 1)
				{
					strTmpRx.Format(_T(" %02X"), g_iSPIDataTmpRx);
					strTmpTx.Format(_T(" %02X"), g_iSPIDataTmpTx);
				}
				else if(g_iDataLegth == 2)
				{
					strTmpRx.Format(_T(" %04X"), g_iSPIDataTmpRx);
					strTmpTx.Format(_T(" %04X"), g_iSPIDataTmpTx);
				}
				else if(g_iDataLegth == 3)
				{
					strTmpRx.Format(_T(" %06X"), g_iSPIDataTmpRx);
					strTmpTx.Format(_T(" %06X"), g_iSPIDataTmpTx);
				}
				else
				{
					strTmpRx.Format(_T(" %08X"), g_iSPIDataTmpRx);
					strTmpTx.Format(_T(" %08X"), g_iSPIDataTmpTx);
				}

				strSPIRx += strTmpRx;
				strSPITx += strTmpTx;
			}			
		}else if('P' == ScriptData[g_iScriptIdx])
		{
			g_iCmdRepeat--;
			
			if(0 == g_iCmdRepeat)
			{
				g_iScriptIdx++;	// End 'P'
				
				g_iScriptStart = g_iScriptIdx;

				// Process Command Header
				g_iScriptIdx++;	// Start 'S'
				g_iCmdRepeat = ScriptData[g_iScriptIdx++];
				g_iScriptIdx+= 2;	// Sikp Idle Time
				
			}
			else
			{
				g_iScriptIdx = g_iScriptStart + 4;
				
			}
		
			// Process Block Header
			g_iDataLegth = (ScriptData[g_iScriptIdx++] + 7) >> 3;
			g_iSPIDataCnt = ScriptData[g_iScriptIdx++];
			g_iDataCnt = 0;	
			i--;			
			
		}else
		{
			// Process Block Header
			g_iDataLegth = (ScriptData[g_iScriptIdx++] + 7) >> 3;
			g_iSPIDataCnt = ScriptData[g_iScriptIdx++];
			g_iDataCnt = 0;
			i--;
		}

		// check Printf
		if(0 == g_iCmdLegth)
		{	
			g_iCmdLegth = CommandLength[SPI0_IDNum];	
			bPrintIDFlag = TRUE;			
		}
		
		if((g_iCmdLegth == strSPILength) || (strSPILength >= 64 && BlockDone))
		{
			if(bPrintIDFlag)
			{
				strSPI0IDItem.Format(_T("%d"), SPI0_IDNum);
				SPI0_IDNum++;
				bPrintIDFlag = FALSE;
			}
			else
				strSPI0IDItem =_T("");								

			
			pListCtl->SetItemColor(nItem, RGB(0,0,255));
			pListCtl->InsertItem(nItem, _T(""));
			pListCtl->SetItemText(nItem, 0, strSPI0IDItem);
			pListCtl->SetItemText(nItem++, 1, _T("MOSI :") + strSPITx);


			pListCtl->SetItemColor(nItem, RGB(255,0,0));							
			pListCtl->InsertItem(nItem, _T(""));
			pListCtl->SetItemText(nItem, 0, strSPI0IDItem);
			pListCtl->SetItemText(nItem++, 1, _T("MISO :") + strSPIRx);
					
			g_iCmdLegth -= strSPILength;
			strSPILength = 0;
			
			strSPIRx = _T("");
			strSPITx = _T("");
		}

	}

	return 0;
}

// For SPI Monitor Display
CString strTextSPIRx0;
CString strTextSPIRx1;
int strLengthSPIM = 0;
UCHAR SPIMBuf[16384] ={0};
UCHAR SPIMAddBuf[16384] ={0};
CString strSPIMIDItem;
int MatchSize = 0, DataSize = 0;
int SPIMAddHead = 0, SPIMAddTail = 0;
int SPIMHead = 0, SPIMTail = 0, SPIMOffset = 0;
int SPIMLengthCur = 0, SPIMLengthNext = 0;
int SPIMLength = 0, SPIMCurIdx = 0, SPIMNextIdx = 0;
BOOL bDataFasterFlag = 0;

UCHAR SPIMBufRx0[0x2000] = {0};
UCHAR SPIMBufRx1[0x2000] = {0};
USHORT SPIMBufTail = 0;
USHORT SPIMBufHead = 0;
USHORT SPIMBufCnt = 0;


USHORT SPIMSS[0x2000] = {0};
USHORT SPIMSSTail = 0;
USHORT SPIMSSHead = 0;
USHORT SPIMSSCnt = 0;

USHORT gSPIMMTmp = 0;

UCHAR ShowSpiMonitor(void)
{
	BOOL bIDPrintFlag = 0;
	
	while(SPIMSSCnt)
	{
		USHORT datalength = SPIMSS[SPIMSSHead];
			
		if(datalength <= SPIMBufCnt)
		{		
			if(datalength > 64)
			{				
				strTextSPIRx0 += byte2wstr(&(SPIMBufRx0[SPIMBufHead]), 64).c_str();
				strTextSPIRx1 += byte2wstr(&(SPIMBufRx1[SPIMBufHead]), 64).c_str();
			
				pListCtl->SetItemColor(nItem, RGB(0,0,255));
				pListCtl->InsertItem(nItem, _T(""));
				if(bIDPrintFlag)
				{
					pListCtl->SetItemText(nItem, 0, _T(""));
				}
				else
				{
					strSPIMIDItem.Format(_T("%d"), IDNum);
					pListCtl->SetItemText(nItem, 0, strSPIMIDItem);
				}
				pListCtl->SetItemText(nItem++, 1, strTextSPIRx1);
				strTextSPIRx1 = _T("MOSI : ");

				pListCtl->SetItemColor(nItem, RGB(255,0,0));
				pListCtl->InsertItem(nItem, _T(""));
				
				if(bIDPrintFlag)
				{
					pListCtl->SetItemText(nItem, 0, _T(""));
				}
				else
				{
					strSPIMIDItem.Format(_T("%d"), IDNum);				
					pListCtl->SetItemText(nItem, 0, strSPIMIDItem);
				}
				
				pListCtl->SetItemText(nItem++, 1, strTextSPIRx0);							
				strTextSPIRx0 = _T("MISO : ");
				
				if((SPIMSS[SPIMSSHead] - 64) > 0)  // not finish
				{	
					SPIMSS[SPIMSSHead] -= 64; 
					SPIMBufHead += 64;
					SPIMBufCnt -= 64;
					bIDPrintFlag = 1;					
				}
			}
			else   // datalength < 64
			{
				strTextSPIRx0 += byte2wstr(&(SPIMBufRx0[SPIMBufHead]), datalength).c_str();
				strTextSPIRx1 += byte2wstr(&(SPIMBufRx1[SPIMBufHead]), datalength).c_str();
			
				pListCtl->SetItemColor(nItem, RGB(0,0,255));
				pListCtl->InsertItem(nItem, _T(""));
				if(bIDPrintFlag)
				{
					pListCtl->SetItemText(nItem, 0, _T(""));					
				}
				else
				{
					strSPIMIDItem.Format(_T("%d"), IDNum);
					pListCtl->SetItemText(nItem, 0, strSPIMIDItem);
				}
				pListCtl->SetItemText(nItem++, 1, strTextSPIRx1);
				strTextSPIRx1 = _T("MOSI : ");

				pListCtl->SetItemColor(nItem, RGB(255,0,0));
				pListCtl->InsertItem(nItem, _T(""));
				
				if(bIDPrintFlag)
				{
					pListCtl->SetItemText(nItem, 0, _T(""));
					bIDPrintFlag = 0;
				}
				else
				{
					strSPIMIDItem.Format(_T("%d"), IDNum);				
					pListCtl->SetItemText(nItem, 0, strSPIMIDItem);
				}
				
				pListCtl->SetItemText(nItem++, 1, strTextSPIRx0);							
				strTextSPIRx0 = _T("MISO : ");
			
				SPIMBufHead += datalength;
				SPIMBufCnt -= datalength;
				SPIMSS[SPIMSSHead] = 0;
				SPIMSSHead ++;
				SPIMSSHead &= 0x1FFF;
				SPIMSSCnt --;	
				
				IDNum++;													
			}			
		}
		else
		{
			//TRACE(_T("SPIMSS : %d %d\n"), SPIMSSTail, SPIMSS[SPIMSSTail]);
			return 1;	// Data Not enough, do nothing!	
		}
				
	}
	
	return 0;
}

UCHAR GetSpiMData(int PacketSize)
{
	int Offset = UsbBuf[UsbBufHead][1] + 4;	
	UCHAR *Watch = &(UsbBuf[UsbBufHead][Offset]);
	int resData = PacketSize - Offset;
	int AddSize = UsbBuf[UsbBufHead][2];
	AddSize += (UsbBuf[UsbBufHead][3] << 8);

	
	if(AddSize == 0xFFFF)
		g_bSPIMException = TRUE;
	BOOL bIDPrintFlag = 0;
	
	while(resData)
	{
		SPIMBufRx0[SPIMBufTail] = (*Watch);
		Watch++;
		SPIMBufRx1[SPIMBufTail] = (*Watch);
		Watch++;
		
		SPIMBufTail++;
		SPIMBufTail &= 0x1FFF;
		
		SPIMBufCnt++;
		
		resData -= 2;
	}
	
	ShowSpiMonitor();

	return 0;
}

UCHAR GetSpiSS(int PacketSize)
{

	int Offset = UsbBuf[UsbBufHead][1] + 4;	
	UCHAR *Watch = &(UsbBuf[UsbBufHead][Offset]);
	int resData = PacketSize - Offset;
	int AddSize = UsbBuf[UsbBufHead][2];
	AddSize += (UsbBuf[UsbBufHead][3] << 8);
	USHORT ssLength;
	
	if(AddSize == 0xFFFF)
		g_bSPIMException = TRUE;

	while(resData)
	{
		ssLength = (*Watch);
		Watch++;

		ssLength |= ((*Watch) << 8);
		Watch++;
		
		SPIMSS[SPIMSSTail] = ((ssLength + 0x4000 - gSPIMMTmp) & 0x3FFF) >> 1;	// FW Index syn.	
		gSPIMMTmp = ssLength;
		
		//TRACE(_T("SPIMSS : %d %d\n"), SPIMSSTail, SPIMSS[SPIMSSTail]);
		
		SPIMSSTail++;
		SPIMSSTail &= 0x1FFF;
		
		SPIMSSCnt++;
		
		resData -= 2;	
	}	

	ShowSpiMonitor();

	return 0;
}

UCHAR g_bADCHeader = 0;
CString StrOut;
extern int ADCShow_Num;
//extern int ScanCnt;
int ScanNum = 0;
UCHAR ShowADC(int PacketSize)
{
	int Offset = UsbBuf[UsbBufHead][1] + 4;	
	int dataCnt = UsbBuf[UsbBufHead][1] + 2;	
	UCHAR *Watch = &(UsbBuf[UsbBufHead][Offset]);
	int resData = PacketSize - Offset;
	int out_tmp = 0, out = 0, Idx[6] = {0}, ADC_Out[6] = {0};
	CString StrFinOut;
	float outv = 0.0;
	
	while(resData)
	{	
	   if(g_bADCHeader % 2 == 0)
		  out_tmp = Watch[g_bADCHeader]; 
	   else
	   {
			out = (Watch[g_bADCHeader] & 0x0F) << 8;
			Idx[(g_bADCHeader >> 1)] = (Watch[g_bADCHeader] & 0xF0) >> 4;

			out_tmp = out + out_tmp;
			ADC_Out[(g_bADCHeader >> 1)] = out_tmp;

			//TRACE(_T("%d %d %d\n"), g_bADCHeader >> 1, Idx[g_bADCHeader >> 1], ADC_Out[g_bADCHeader >> 1]);
	   }
	   
	   g_bADCHeader++;
	   resData--;

	   if(g_bADCHeader == (2 * ADCShow_Num))
	   {		
		   g_bADCHeader = 0;
			for(int i = 0; i < ADCShow_Num; i++)
			{
				outv = (ADC_Out[i] + 1) / (1024.0);
				outv = outv * InputVol;

				if(i == 5)
					StrOut.Format(_T("ADC %1d : %.2fv"), Idx[i], outv);
				else
					StrOut.Format(_T("ADC %1d : %.2fv  "), Idx[i], outv);
					
				StrFinOut += StrOut;
				StrFinOut += _T(" ");			
			}
			
			pListCtl->SetItemColor(nItem, RGB(0,0,0));
			StrOut.Format(_T("%d"), ScanNum++);
			pListCtl->InsertItem(nItem, StrOut);
			pListCtl->SetItemText(nItem++, 1, StrFinOut);	
			StrFinOut = _T("");
			StrOut = _T("");
	   }
	}

	return 0;
}

bool InitADC()
{
	g_bADCHeader = 0;
	ScanNum = 0;
	StrOut = _T("");
	return 0;
}


UCHAR g_u8PWMHeader = 0;
CString StrOut_tmp;
CString StrOut0, StrOut1;
int out_Byte1 = 0, out_Byte2 = 0, out_Byte3 = 0;
int PWMSize = 0, FallTail0 = 0, FallTail1 = 0, FallHead0 = 0, FallHead1 = 0;
int PWMStatus = 0;
UCHAR bRiseChg = 0;
//int sCurPar[3] = {0};
//int sPrePar[2][2][3] = {0};   // [channel] [fall/rise] [3byte] 
/*
bool InitPWM()
{
	g_bPWMException = 0;
	PWMSize = 0;
	
	g_u8PWMHeader = 0;

	StrOut = _T("");
	StrOut_tmp = _T("");

	
    edge_type0 = edge_type1 = 0;
	PWMStatus = STATE_START;
	out_Byte1 = out_Byte2 = out_Byte3 = 0;
	FallTail0 = FallTail1 = FallHead0 = FallHead1 = 0;
	run_PWMCState(0xFF, 0);    ///< DataType 0xFF means to clear static parameters.

	return 0;
}
*/
UCHAR run_PWMCState(UCHAR DataType, int Data)
{
    static UCHAR sChannel = 0xFF;
	static int sCurPar[3] = {0};
	static int sPrePar[2][2][3] = {0};   // [channel] [fall/rise] [3byte] 

	if (DataType == 0xFF)
	{
	    memset(sCurPar,0,sizeof(sCurPar));
	    memset(sPrePar,0,sizeof(sPrePar));
		return 0;
	}
	
	switch(DataType)
	{
		case 0:
		{
            if ((Data == 0) ||  (Data == 1))  ///< Channel 0 
            {
			    if (Data == 0) 
  				    Edge0 = 1;
				else //if (Data == 1)
				    Edge0 = 0;
					
			    sChannel = 0;
				sCurPar[0] = Data;
            }
			else //if ((Data == 2) ||  (Data == 3))  ///< Channel 1
			{
		        if (Data == 2) 
  				    Edge1 = 1;
				else //if (Data == 3)
				    Edge1 = 0;
					
			    sChannel = 1;
				sCurPar[0] = Data;
			}
		}	
		break;
						
		case 1:
			sCurPar[1] = Data;
		    break;
		case 2:
		{
			if (sChannel == 0)
			{
			   sCurPar[2] = Data;
			   
			   
			   if (Edge0 == 0)
			   {
					if ((sCurPar[0] != sPrePar[0][1][0]) || (sCurPar[1] != sPrePar[0][1][1]) || (sCurPar[2] != sPrePar[0][1][2]))
					{
						FallWidth0 = Data;
						sPrePar[0][1][0] = sCurPar[0];	
						sPrePar[0][1][1] = sCurPar[1];	
						sPrePar[0][1][2] = sCurPar[2];
					}
					else
					{
						return 0;
					}
			   }
			   else // if (Edge0 == 1)
			   {
					if ((sCurPar[0] != sPrePar[0][0][0]) || (sCurPar[1] != sPrePar[0][0][1]) || (sCurPar[2] != sPrePar[0][0][2]))
					{
						RiseWidth0 = Data;
						sPrePar[0][0][0] = sCurPar[0];	
						sPrePar[0][0][1] = sCurPar[1];	
						sPrePar[0][0][2] = sCurPar[2];
					}
					else
					{
						return 0;
					}						
			   }
			   
			   if(edge_type0 == 2)
					duty0 = float(FallWidth0) / float(RiseWidth0 + FallWidth0);
			   
			   PWMStatus |= 0x01;
			   return 0x01;
			}
			else if (sChannel == 1)
			{
			   sCurPar[2] = Data;
			   //if ((sCurPar[0] != sPrePar[1][0]) || (sCurPar[1] != sPrePar[1][1]) || (sCurPar[2] != sPrePar[1][2]))
			   {
				   if (Edge1 == 0)
				   {
						if ((sCurPar[0] != sPrePar[1][1][0]) || (sCurPar[1] != sPrePar[1][1][1]) || (sCurPar[2] != sPrePar[1][1][2]))
						{
						    FallWidth1 = Data;
                            sPrePar[1][1][0] = sCurPar[0];	
                            sPrePar[1][1][1] = sCurPar[1];	
                            sPrePar[1][1][2] = sCurPar[2];
						}
						else
						{
						    return 0;
						}
				   }
				   else // if (Edge1 == 1)
				   {
					    if ((sCurPar[0] != sPrePar[1][0][0]) || (sCurPar[1] != sPrePar[1][0][1]) || (sCurPar[2] != sPrePar[1][0][2]))
						{
						    RiseWidth1 = Data;
                            sPrePar[1][0][0] = sCurPar[0];	
                            sPrePar[1][0][1] = sCurPar[1];	
                            sPrePar[1][0][2] = sCurPar[2];
						}
						else
						{
						    return 0;
						}
				   }
				   
				   if(edge_type1 == 2)
						duty1 = float(FallWidth1) / float(RiseWidth1 + FallWidth1);
				   
				   PWMStatus |= 0x02;
				   return 0x02;
			   }
			}
		}
		    break;
        default:
		    break;
	}
	return 0;
}


bool InitPWM()
{
	g_bPWMException = 0;
	PWMSize = 0;
	
	g_u8PWMHeader = 0;

	//StrOut = _T("");
	StrOut_tmp = _T("");
	StrOut0 = StrOut1 =  _T("");
	bRiseChg = 0;
    edge_type0 = edge_type1 = 0;
	PWMStatus = STATE_START;
	out_Byte1 = out_Byte2 = out_Byte3 = 0;
	FallTail0 = FallTail1 = FallHead0 = FallHead1 = 0;
	run_PWMCState(0xFF, 0);    ///< DataType 0xFF means to clear static parameters.

	return 0;
}

UCHAR ShowPWM(int PacketSize)
{
	int Offset = UsbBuf[UsbBufHead][1] + 4;	
	UCHAR *Watch = &(UsbBuf[UsbBufHead][Offset]);
	int resData = PacketSize - Offset;
	
	int AddSize = UsbBuf[UsbBufHead][2];
	AddSize += (UsbBuf[UsbBufHead][3] << 8);
	
	if(AddSize == 0xFFFF)
		g_bPWMException = TRUE;
		
	if(g_bPWMException)
		return 1;	

	if(nItem > 20000)
	{
		pListCtl->DeleteAllItems();
		nItem = 0;
	}
	
	int TmpIdx = 0;
	UCHAR bRet = 0;	
	
	CString Str0, Str1;
	TmpIdx = g_u8PWMHeader % 3;
	
	for(int i = 0; i < resData; i++)
	{	 
	   if(TmpIdx == 0)
	   {
		  if(Watch[i] > 7)  // because underflow
		  {
			 //Watch[i] = Watch[i];
			 //TRACE(_T("%d %d %d %d %d %d\n"), UsbBuf[UsbBufHead][0], UsbBuf[UsbBufHead][1], UsbBuf[UsbBufHead][2], UsbBuf[UsbBufHead][3], UsbBuf[UsbBufHead][4], UsbBuf[UsbBufHead][5]);
			 continue;
		  }	
		  out_Byte1 = (Watch[i] & 0x03); 
		  switch(out_Byte1)
		  {
			case 0:
				StrOut_tmp = _T("PWM 0 Rising  ");
			break;
			
			case 1:
				StrOut_tmp = _T("PWM 0 Falling ");
			break;
			
			case 2:
				StrOut_tmp = _T("PWM 1 Rising  ");
			break;
			
			case 3:
				StrOut_tmp = _T("PWM 1 Falling ");
			break;
		  }
		  
		   bRet = run_PWMCState(0, out_Byte1);

		  //out_Byte1 = (Watch[i] & 0x04);
		  if(Watch[i] & 0x04)
		  	StrOut_tmp += _T("Underflow");	
	   }
	   else if(TmpIdx == 1)
	   {
			out_Byte2 = Watch[i];
			bRet = run_PWMCState(1, out_Byte2);
	   }
	   else if(TmpIdx == 2)
	   {			
			out_Byte3 = Watch[i] << 8;
			out_Byte3 = out_Byte2 + out_Byte3;
			out_Byte3 = 65535 - out_Byte3;

			out_Byte3 = out_Byte3 * 125 / 1000;

			bRet = run_PWMCState(2, out_Byte3);
	   }
	  
	   g_u8PWMHeader++;
	   TmpIdx = g_u8PWMHeader % 3;	   
	   if(TmpIdx == 0)
	   {
		   CString StrNum;
		   //if (bRet)
		   switch(bRet)
		   {		
				case 1:
				if(edge_type0 == 2)
				{
					if(out_Byte1 == 1)
					{						
						pListCtl->SetItemColor(nItem, RGB(0,0,0));
						Str0.Format(_T("%s %s : %4d "),StrOut0, StrOut_tmp, out_Byte3);		
						pListCtl->InsertItem(nItem, _T(""));
							
						StrNum.Format(_T("%d"),nItem);
						pListCtl->SetItemText(nItem, 0, StrNum);
						pListCtl->SetItemText(nItem++, 1, Str0);
						
						StrOut0 = Str0 = _T("");
						bRiseChg &= 0xFE;    ////< Clear bit 0.
						//DataReady0 = 0;
					}
					else if(out_Byte1 == 0)
					{
						//DataReady0++;
						StrOut0.Format(_T("%s : %4d      "),StrOut_tmp, out_Byte3);
						bRiseChg |= 1;
					}
				}
				else 
				{
					pListCtl->SetItemColor(nItem, RGB(0,0,0));
					StrOut0.Format(_T("%s : %4d "),StrOut_tmp, out_Byte3);		
					pListCtl->InsertItem(nItem, _T(""));
						
					StrNum.Format(_T("%d"),nItem);
					pListCtl->SetItemText(nItem, 0, StrNum);
					pListCtl->SetItemText(nItem++, 1, StrOut0);
					
					StrOut0 = _T("");
				}
				break;
		   
		        case 2:			
				if(edge_type1 == 2)
				{
					if(out_Byte1 == 3)
					{						
						pListCtl->SetItemColor(nItem, RGB(0,0,0));
						Str1.Format(_T("%s %s : %4d "),StrOut1, StrOut_tmp, out_Byte3);		
						pListCtl->InsertItem(nItem, _T(""));
							
						StrNum.Format(_T("%d"),nItem);
						pListCtl->SetItemText(nItem, 0, StrNum);
						pListCtl->SetItemText(nItem++, 1, Str1);
						
						StrOut1 = Str1 = _T("");
						bRiseChg &= 0xFD;    ////< Clear bit 1.
					}
					else if(out_Byte1 == 2)
					{
						StrOut1.Format(_T("%s : %4d      "),StrOut_tmp, out_Byte3);
						bRiseChg |= 2;
					}
				}
				else
				{
					pListCtl->SetItemColor(nItem, RGB(0,0,0));
					StrOut1.Format(_T("%s : %4d "),StrOut_tmp, out_Byte3);		
					pListCtl->InsertItem(nItem, _T(""));
						
					StrNum.Format(_T("%d"),nItem);
					pListCtl->SetItemText(nItem, 0, StrNum);
					pListCtl->SetItemText(nItem++, 1, StrOut1);
					
					StrOut1 = _T("");
				}
			    break;
				
		        case 0:
				{
				    if (out_Byte1 % 2)    ///< Means falling
					{
					    if (bRiseChg & 0x01)  ///< channel 0 rise changed.
						{
						    /// Draw
							pListCtl->SetItemColor(nItem, RGB(0,0,0));
							Str0.Format(_T("%s %s : %4d "),StrOut0, StrOut_tmp, out_Byte3);		
							pListCtl->InsertItem(nItem, _T(""));
								
							StrNum.Format(_T("%d"),nItem);
							pListCtl->SetItemText(nItem, 0, StrNum);
							pListCtl->SetItemText(nItem++, 1, Str0);
							
							StrOut0 = Str0 = _T("");
						    bRiseChg &= 0xFE;    ////< Clear bit 0.
						}
						else if (bRiseChg & 0x02)    ///< channel 1 rise changed.
						{
						    /// Draw
							Str1.Format(_T("%s %s : %4d "),StrOut1, StrOut_tmp, out_Byte3);		
							pListCtl->InsertItem(nItem, _T(""));
								
							StrNum.Format(_T("%d"),nItem);
							pListCtl->SetItemText(nItem, 0, StrNum);
							pListCtl->SetItemText(nItem++, 1, Str1);
							
							StrOut1 = Str1 = _T("");

						    bRiseChg &= 0xFD;    ////< Clear bit 1.
						}
					}
				}
				    break;
				default:
				    /// It should not happen.
				    break;
		   }
           g_u8PWMHeader -= 3;
		   out_Byte1 = out_Byte2 = out_Byte3 = 0;
	   }
	}	
	return 0;
}


volatile unsigned char g_u8ModId;
volatile unsigned char g_u8Padding;
volatile unsigned char *g_pu8Watch;
volatile unsigned char *g_pu8UsbBuf;
volatile unsigned char g_u8DataCnt;
volatile unsigned char g_u8BufSize;

// Usb Packet = 4 Byte Header + Padding + Data
// 4 Byte Header = Module ID(1) + Padding Cnt(1) + FW Buffer Usage(2)

// Retreve Header Information from USB packet
void RetrieveHeader(int iUsbIdx)
{
	g_u8ModId = UsbBuf[iUsbIdx][0];
	g_u8Padding = UsbBuf[iUsbIdx][1];

	int Offset =  g_u8Padding + 4;
	g_pu8UsbBuf = &(UsbBuf[iUsbIdx][0]);
	g_pu8Watch =  g_pu8UsbBuf + Offset;
	g_u8BufSize = UsbBufSize[iUsbIdx];
	g_u8DataCnt = g_u8BufSize - Offset;
}

void ShowDefault()
{
	CString strHeader = byte2wstr((unsigned char*)g_pu8UsbBuf, 4).c_str();
	CString strTemp = byte2wstr((unsigned char*)g_pu8Watch, g_u8DataCnt).c_str();
	pListCtl->InsertItem(nItem, _T(""));
	pListCtl->SetItemText(nItem, 0, strHeader);
	pListCtl->SetItemText(nItem++, 1, strTemp);
}


UINT ListWriteThread(LPVOID pVar)
{
	CNuBridgeDlg *pDlg = (CNuBridgeDlg *)pVar;
	pListCtl = &(pDlg->m_MAIN_LIST);

	int ret = 0;
	
	InitSpiMonitorSts();
	InitSpi0MasterSts();
	InitADC();
	InitPWM();
	UsbBufHead = 0;

	while(g_bThreadContinue)
	{
		RetrieveHeader(UsbBufHead);
	
		ret = UsbBufSize[UsbBufHead];
		nItem = pDlg->m_MAIN_LIST.GetItemCount(); 
		if (ret == 0)
		{
			Sleep(1);
			continue;
		}

		if (1)
		{

			int ModuleId = UsbBuf[UsbBufHead][0];
			
			int TimeOutDuration = 100; // mini sec
			
			switch(ModuleId)
			{

				case MODE_SPI_MONITOR:
					pDlg->KillTimer(MODE_SPI_MONITOR);
					
					if(g_bSPIMException)
						pDlg->SetTimer(MODE_SPI_MONITOR, 2000, NULL);
					
					GetSpiMData(ret);
				break;

				case MODE_I2C_MONITOR:
				
					pDlg->KillTimer(MODE_I2C_MONITOR);				
					
				#if(SOFTWARE_I2C_MONITOR == GPIO_MONITOR1)
					ShowI2cMonitor(ret);					
				#elif(SOFTWARE_I2C_MONITOR == GPIO_MONITOR2)
					ShowI2cMonitor2();				
				#endif
					
					pDlg->SetTimer(MODE_I2C_MONITOR, TimeOutDuration, NULL);
			
				break;

				case MODE_I2C0_MASTER:
					
					ShowI2c0Master(ret);
					
				break;
				
				case MODE_SPI0_MASTER:
				case MODE_SPI2_MASTER:
				
					ShowSpi0Master(ret);			
					
				break;

				case MODE_ADC:
				
					ShowADC(ret);								
					
				break;

				case MODE_PWM:			
				
					pDlg->KillTimer(MODE_PWM);
					
					if(g_bPWMException)
						pDlg->SetTimer(MODE_PWM, 200, NULL);	
					
					ShowPWM(ret);								
				
				break;

								
				case 0xFF:
					GetSpiSS(ret);
				break;
				default:
					ShowDefault();
				break;
			}
	
			UsbBufSize[UsbBufHead] = 0;
			UsbBufHead++;
			UsbBufHead &= BOUND;
			pDlg->m_MAIN_LIST.SetRedraw(TRUE);
		}
	}

	UsbBufHead = 0;
	memset((void *)UsbBufSize, 0, BLOCK_CNT);
	
	return 0x34;
}

bool InitI2cMasterSts()
{
	bI2cIDPrint = 0;
	g_bI2cHeader = 0;
	g_I2cActiveCmd = 0;
	g_I2cScriptPointer = 1;
	return 0;
}

bool InitSpi0MasterSts()
{
	SPI0_IDNum = 0;
	bPrintIDFlag = TRUE;	
	// Access Script Data
	g_iScriptIdx = 0;
	g_iCmdLegth = 0;
	
	return 0;
}

bool InitSpiMonitorSts()
{
	g_bSPIMException = 0;
	SPIMCurIdx = 0;

	SPIMAddHead = 0;
	SPIMAddTail = 0;
	SPIMHead = 0;
	SPIMTail = 0;

	memset(SPIMBufRx0, 0, sizeof(UCHAR)*8192);
	memset(SPIMBufRx1, 0, sizeof(UCHAR)*8192);
	memset(SPIMSS, 0, sizeof(USHORT)*8192);
	
	SPIMSSTail = 0;
	SPIMSSHead = 0;
	SPIMSSCnt = 0;

	gSPIMMTmp = 0;
	SPIMBufTail = 0;
	SPIMBufHead = 0;
	SPIMBufCnt = 0;		
	
	return 0;
}



UCHAR InitThreadData()
{
	// Reset Thread queue pointer
	// Move queue pointer setting to each thread

	// I2C Monitor status
	ch = 0;
	bitCnt = 0;
	Ack = 0;
	Addr = 0;
	strTextI2C = _T("");
	
	// I2C Master status
	g_bI2cHeader = 0;
	g_I2cActiveCmd = 0;
	g_I2cScriptPointer = 1;
	
	return 0;	
}

void TimeOut(int nIDEvent)
{
	switch(nIDEvent)
	{
		case MODE_I2C_MONITOR:
			if(strTextI2C != _T(""))
			{
				pListCtl->InsertItem(nItem, _T(""));
				pListCtl->SetItemText(nItem++, 1, strTextI2C);

				strTextI2C = _T("");
			}
		break;
		
		case 0xFF:
				pListCtl->InsertItem(nItem, _T(""));
				pListCtl->SetItemText(nItem++, 1, strTextHex);
				strTextHex = _T("");
							
				strLengthHex = 0;
		break;
		
		default:
		break;
	}
}

 
 /**
  * @brief  GPCDF main interrupt function.
  *             store received data in g_u8RxBuf, update related queue pointer and queue size
  *             received pattern is 2-byte pattern explained as follows
  *             	0x00 0xXX	: Data byte 0xXX	(Low Byte = 0 means data)
  *             	'S'	 0x00	: I2C Start Bit		(non-zero Low Byte is used for specific pattern)
  *             	'P'	 0x00	: I2C Stop Bit
  *             	'R'	 0x00	: I2C Read Bit
  *             	'W'	 0x00	: I2C Write Bit
  *             	0x01 0xXX	: I2C 7-Bit Slave Addr.
  *             	0x02 0x01	: I2C NACK Bit
  *             	0x02 0x00	: I2C ACK Bit
  * @param  None.
  * @retval None.
  */ 
UCHAR ShowI2cMonitor2()
{
	unsigned char ch;
	
	for(int i = 0; i < g_u8DataCnt; i+=2)
	{
		ch = g_pu8Watch[i];
		switch(ch)
		{
			case 'S':
				strTextI2C.AppendChar(_T('['));
				strTextI2C.AppendChar(_T('S'));
				strTextI2C.AppendChar(_T(']'));
				strTextI2C.AppendChar(_T(' '));
			break;
			
			case 'P':
				strTextI2C.AppendChar(_T('['));
				strTextI2C.AppendChar(_T('P'));
				strTextI2C.AppendChar(_T(']'));
				strTextI2C.AppendChar(_T(' '));
				pListCtl->SetItemColor(nItem, RGB(0,0,0));
				pListCtl->InsertItem(nItem, _T(""));
				pListCtl->SetItemText(nItem++, 1, strTextI2C);
				strTextI2C = _T("");
			break;
			
			case 'R':
				strTextI2C.AppendChar(_T('"'));
				strTextI2C.AppendChar(_T('R'));
				strTextI2C.AppendChar(_T(' '));
			break;
			
			case 'W':
				strTextI2C.AppendChar(_T('"'));
				strTextI2C.AppendChar(_T('W'));
				strTextI2C.AppendChar(_T(' '));
			break;
	
			
			case 0:		// Data			
			case 1:		// Address
				ch = g_pu8Watch[i+1];
				strTextI2C.AppendChar(THexTable[(ch & 0xF0) >> 4]);
				strTextI2C.AppendChar(THexTable[(ch & 0x0F)]);						
				strTextI2C.AppendChar(_T(' '));
			break;
			
			// Ack
			case 2:
			break;
			
			default:
			break;
		}
	}
	return 0;
}