/**************************************************************************//**
 * @file     SPI_Monitor_Dlg.cpp
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 13/12/03 11:27a $
 * @brief    NuBridge SPI Monitor Main Dialog
 *
 * @note
 * Copyright (C) 2013 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
// SPI_Monitor_Dlg.cpp : implementation file
//

#include "stdafx.h"
#include "NuBridge.h"
#include "SPI_Monitor_Dlg.h"
#include "NuDlgMessage.h"
#include "NuBridgeDlg.h"

#include "NuWidget.h"

// SPI_Monitor_Dlg dialog

IMPLEMENT_DYNAMIC(SPI_Monitor_Dlg, CDialog)

SPI_Monitor_Dlg::SPI_Monitor_Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(SPI_Monitor_Dlg::IDD, pParent)
{
	m_bInit = FALSE;
	m_CtrlCnt = 0;
}

SPI_Monitor_Dlg::~SPI_Monitor_Dlg()
{
}

void SPI_Monitor_Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_BTN_SPIM_RUN, m_SPIMbtns[0]);
	DDX_Control(pDX, IDC_BTN_SPIM_DISABLE, m_SPIMbtns[1]);
	DDX_Control(pDX, IDC_BTN_SPIM_PARM, m_SPIMbtns[2]);
}


BEGIN_MESSAGE_MAP(SPI_Monitor_Dlg, CDialog)
	ON_BN_CLICKED(IDC_BTN_SPIM_RUN, &SPI_Monitor_Dlg::OnBnClickedBtnSpiRun)
	ON_BN_CLICKED(IDC_BTN_SPIM_PARM, &SPI_Monitor_Dlg::OnBnClickedBtnSpimParm)
	ON_BN_CLICKED(IDC_BTN_SPIM_DISABLE, &SPI_Monitor_Dlg::OnBnClickedBtnSpimDisable)

	ON_NOTIFY_EX(TTN_NEEDTEXT, 0, &SPI_Monitor_Dlg::SetTipText)
END_MESSAGE_MAP()

BOOL SPI_Monitor_Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// TODO:  Add extra initialization here
	InitDialogWnd(FALSE);
	EnableToolTips(TRUE);

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

// SPI_Monitor_Dlg message handlers

void SPI_Monitor_Dlg::OnBnClickedBtnSpiRun()
{
	InitSpiMonitorSts();
	// TODO: Add your control notification handler code here
	if(NuUsb.WinUsbNumber > 0)
	{
		GetParent()->SendMessage(WM_LOCK_SCREEN, (WPARAM)1);
		InitDialogWnd(FALSE);
	}
	else
	{
		AfxMessageBox(_T("Nu-Bridge Not Found"));
		InitDialogWnd(FALSE);
		return;
	}
}
void SPI_Monitor_Dlg::OnBnClickedBtnSpimParm()
{
	// TODO: Add your control notification handler code here
	SPI_Para_Dlg SPIMParaDlg;
	SPIMParaDlg.m_iSpiMode = MODE_SPI_MONITOR;
	
	if(NuUsb.WinUsbNumber > 0)
	{
		SPIMParaDlg.DoModal();
	}
	else
	{
		AfxMessageBox(_T("Nu-Bridge Not Found"));
	}
}
void SPI_Monitor_Dlg::InitDialogWnd(bool bShow)
{
	// Disable All Member
	if(!m_bInit)
	{
		int i = 0;

		m_SPIMbtns[i].SetBitmaps(IDB_BITMAP_BTN_MODE, RGB(0, 255, 0)); 
		m_SPIMbtns[i++].DrawTransparent();

		m_SPIMbtns[i].SetBitmaps(IDB_BITMAP_BTN_STOP, RGB(0, 255, 0)); 
		m_SPIMbtns[i++].DrawTransparent();

		m_SPIMbtns[i].SetBitmaps(IDB_BITMAP_BTN_CONFIG, RGB(0, 255, 0)); 
		m_SPIMbtns[i++].DrawTransparent();


		m_bInit = TRUE;
		m_CtrlCnt = i;
	}

	if(NuUsb.WinUsbNumber == 0)
	{	
		for(int i = 0; i < m_CtrlCnt; i++)
		{
			m_SPIMbtns[i].EnableWindow(FALSE);
		}
	}
	else
	{
		m_SPIMbtns[0].EnableWindow(bShow);	
		
		for(int i = 1; i < m_CtrlCnt; i++)
		{
			m_SPIMbtns[i].EnableWindow(!bShow);
		}
	}
	
}

void SPI_Monitor_Dlg::SPIMSwitchLang(HINSTANCE tmpDll, int LanguageIdx)
{
	int i;
	if(LanguageIdx == 1)
	{
        i = 0;
		m_SPIMbtns[i].SetBitmaps(IDB_BITMAP_CHT_BTN_MODE, RGB(0, 255, 0)); 
		m_SPIMbtns[i++].DrawTransparent();

		m_SPIMbtns[i].SetBitmaps(IDB_BITMAP_CHT_BTN_STOP, RGB(0, 255, 0)); 
		m_SPIMbtns[i++].DrawTransparent();

		m_SPIMbtns[i].SetBitmaps(IDB_BITMAP_CHT_BTN_CONFIG, RGB(0, 255, 0)); 
		m_SPIMbtns[i++].DrawTransparent();
	}
	else if(LanguageIdx == 2)
	{
		i = 0;
		m_SPIMbtns[i].SetBitmaps(IDB_BITMAP_CHS_BTN_MODE, RGB(0, 255, 0)); 
		m_SPIMbtns[i++].DrawTransparent();

		m_SPIMbtns[i].SetBitmaps(IDB_BITMAP_CHS_BTN_STOP, RGB(0, 255, 0)); 
		m_SPIMbtns[i++].DrawTransparent();

		m_SPIMbtns[i].SetBitmaps(IDB_BITMAP_CHS_BTN_CONFIG, RGB(0, 255, 0)); 
		m_SPIMbtns[i++].DrawTransparent();
	}
	else
	{
		i = 0;
		m_SPIMbtns[i].SetBitmaps(IDB_BITMAP_BTN_MODE, RGB(0, 255, 0)); 
		m_SPIMbtns[i++].DrawTransparent();

		m_SPIMbtns[i].SetBitmaps(IDB_BITMAP_BTN_STOP, RGB(0, 255, 0)); 
		m_SPIMbtns[i++].DrawTransparent();

		m_SPIMbtns[i].SetBitmaps(IDB_BITMAP_BTN_CONFIG, RGB(0, 255, 0)); 
		m_SPIMbtns[i++].DrawTransparent();
	}
	LangDll = tmpDll;
}

void SPI_Monitor_Dlg::OnBnClickedBtnSpimDisable()
{
	// TODO: Add your control notification handler code here
	
	if(NuUsb.WinUsbNumber > 0)
	{
		GetParent()->SendMessage( WM_LOCK_SCREEN, (WPARAM)0);
		InitDialogWnd(TRUE);
	}
	else
	{
		AfxMessageBox(_T("Nu-Bridge Not Found"));
		InitDialogWnd(FALSE);
		return;
	}
}

void SPI_Monitor_Dlg::OnCancel()
{
	// TODO: Add your specialized code here and/or call the base class

	//CDialog::OnCancel();
}

void SPI_Monitor_Dlg::OnOK()
{
	// TODO: Add your specialized code here and/or call the base class

	//CDialog::OnOK();
}

BOOL SPI_Monitor_Dlg::SetTipText(UINT id, NMHDR *pTTTStruct, LRESULT *pResult)
{
	TOOLTIPTEXT *pTTT = (TOOLTIPTEXT *)pTTTStruct;          
	UINT nID = pTTTStruct->idFrom; 

	if (pTTT->uFlags & TTF_IDISHWND)  
	{
		nID = ::GetDlgCtrlID((HWND)nID);
		if (NULL == nID)
			return FALSE;
		
		switch(nID)
		{
			case IDC_BTN_SPIM_PARM:
			{
				if(LangDll == NULL)
					pTTT->lpszText = _T("Config SPI Monitor parameter.");
				else
				{
					ToolTipStr.LoadString(LangDll, IDS_TOOLTIP_SPIM_CONFIG);
					pTTT->lpszText = (LPWSTR)((LPCTSTR)ToolTipStr);
				}
			}	
			break;  
				
			case IDC_BTN_SPIM_RUN:
			{
				if(LangDll == NULL)
					pTTT->lpszText = _T("Start SPI Monitor.");
				else
				{
					ToolTipStr.LoadString(LangDll, IDS_TOOLTIP_SPIM_MODE);
					pTTT->lpszText = (LPWSTR)((LPCTSTR)ToolTipStr);
				}
			}
			break;
			
			case IDC_BTN_SPIM_DISABLE:
			{
				if(LangDll == NULL)
					pTTT->lpszText = _T("Stop SPI Monitor.");
				else
				{
					ToolTipStr.LoadString(LangDll, IDS_TOOLTIP_SPIM_DISABLE);
					pTTT->lpszText = (LPWSTR)((LPCTSTR)ToolTipStr);
				}
			}	
			break;

			default:
				break;
		}

		return TRUE;
	}  
	return FALSE;
}
