/**************************************************************************//**
 * @file     SPI_Para_Dlg.cpp
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 13/12/03 11:27a $
 * @brief    NuBridge SPI Parameter Setting Dialog 
 *
 * @note
 * Copyright (C) 2013 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
// SPI_Para_Dlg.cpp : implementation file
//

#include "stdafx.h"
#include "NuBridge.h"
#include "SPI_Para_Dlg.h"


// SPI_Para_Dlg dialog
extern int m_iLanguage;
IMPLEMENT_DYNAMIC(SPI_Para_Dlg, CDialog)

SPI_Para_Dlg::SPI_Para_Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(SPI_Para_Dlg::IDD, pParent)
{

}

SPI_Para_Dlg::~SPI_Para_Dlg()
{
}

void SPI_Para_Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_COMBO_SPI_CLK, m_COMBO_SPI_CLK);
	DDX_Control(pDX, IDC_COMBO_SPI_MODE, m_COMBO_SPI_TYPE);
	DDX_Control(pDX, IDC_COMBO_SPI_ORDER, m_COMBO_SPI_ORDER);
	DDX_Control(pDX, IDC_COMBO_SPI_SS, m_COMBO_SPI_SS);
	DDX_Control(pDX, IDC_COMBO_SPI_BITS, m_COMBO_SPI_BITS);

	DDX_Control(pDX, IDC_BTN_SPI_GET, m_SPIParmbtns[0]);
	DDX_Control(pDX, IDC_BTN_SPI_SET, m_SPIParmbtns[1]);
	DDX_Control(pDX, IDC_STATIC_SPI0_TYPEIMG, m_SPI0_TypeIMG);
}


BEGIN_MESSAGE_MAP(SPI_Para_Dlg, CDialog)
	ON_BN_CLICKED(IDC_BTN_SPI_GET, &SPI_Para_Dlg::OnBnClickedBtnSpiGet)
	ON_BN_CLICKED(IDC_BTN_SPI_SET, &SPI_Para_Dlg::OnBnClickedBtnSpiSet)
	ON_CBN_SELCHANGE(IDC_COMBO_SPI_MODE, &SPI_Para_Dlg::OnCbnSelchangeComboSpiMode)
	ON_CBN_SELCHANGE(IDC_COMBO_SPI_SS, &SPI_Para_Dlg::OnCbnSelchangeComboSpiSs)
END_MESSAGE_MAP()


// SPI_Para_Dlg message handlers

BOOL SPI_Para_Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// TODO:  Add extra initialization here
	SPIParmSwitchLang(m_iLanguage);

	if(m_iSpiMode != MODE_SPI_MONITOR)
	{
		SetWindowText(_T("SPI Master Setting"));
	
		m_COMBO_SPI_CLK.AddString(_T("1M   Hz"));
		m_COMBO_SPI_CLK.AddString(_T("2M   Hz"));
		m_COMBO_SPI_CLK.AddString(_T("3M   Hz"));
		m_COMBO_SPI_CLK.AddString(_T("4M   Hz"));
		m_COMBO_SPI_CLK.AddString(_T("4.5M Hz"));
		m_COMBO_SPI_CLK.AddString(_T("6M   Hz"));
		m_COMBO_SPI_CLK.AddString(_T("7.2M Hz"));
		m_COMBO_SPI_CLK.AddString(_T("9M   Hz"));
		m_COMBO_SPI_CLK.AddString(_T("12M  Hz"));
	}
	else
	{
		SetWindowText(_T("SPI Monitor Setting"));

		m_COMBO_SPI_CLK.AddString(_T("12M  Hz"));
		m_COMBO_SPI_CLK.SetCurSel(0); 
		m_COMBO_SPI_CLK.EnableWindow(FALSE);
	}

	m_COMBO_SPI_TYPE.AddString(_T("Type 0"));
	m_COMBO_SPI_TYPE.AddString(_T("Type 1"));
	m_COMBO_SPI_TYPE.AddString(_T("Type 2"));
	m_COMBO_SPI_TYPE.AddString(_T("Type 3"));

	m_COMBO_SPI_ORDER.AddString(_T("LSB First"));
	m_COMBO_SPI_ORDER.AddString(_T("MSB First"));

	m_COMBO_SPI_BITS.AddString(_T("8"));
	m_COMBO_SPI_BITS.SetCurSel(0); 

	m_COMBO_SPI_SS.AddString(_T("Low"));
	m_COMBO_SPI_SS.AddString(_T("High"));

	OnBnClickedBtnSpiGet();
	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void SPI_Para_Dlg::OnBnClickedBtnSpiGet()
{
	// TODO: Add your control notification handler code here
	UCHAR PARM = NuUsb.NU_GetSPIParam(m_iSpiMode);
	
	m_COMBO_SPI_TYPE.SetCurSel(PARM & 0x03);

	HBITMAP hbmp;
	switch (PARM & 0x03)
	{
		case 0:
		{
			if(PARM & 0x08)
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE0_SS1), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
			else
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE0_SS0), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
		}
		break;
		case 1:
		{
			if(PARM & 0x08)
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE1_SS1), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
			else
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE1_SS0), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
		}
		break;
		case 2:
		{
			if(PARM & 0x08)			
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE2_SS1), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
			else
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE2_SS0), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
		}
		break;
		case 3:
		{
			if(PARM & 0x08)
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE3_SS1), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
			else
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE3_SS0), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
		}
		break;
	}
	m_SPI0_TypeIMG.SetBitmap(hbmp);

	if(PARM & 0x04)
		m_COMBO_SPI_ORDER.SetCurSel(1);
	else
		m_COMBO_SPI_ORDER.SetCurSel(0);

	if(PARM & 0x08)
		m_COMBO_SPI_SS.SetCurSel(1);
	else
		m_COMBO_SPI_SS.SetCurSel(0);


	PARM = (PARM & 0xF0) >> 4;
	
	if(PARM >= 11) // 12M Hz
		m_COMBO_SPI_CLK.SetCurSel(8); 
	else
	{
		switch(PARM)
		{
			case(0): // 1M Hz
			case(1): // 2M Hz
			case(2): // 3M Hz
			case(3): // 4M Hz			
			case(4): // 4.5M Hz
			case(5): // 6M Hz
				m_COMBO_SPI_CLK.SetCurSel(PARM); 
			break;

			case(6): // 6M Hz
				m_COMBO_SPI_CLK.SetCurSel(5); 
			break;
						
			case(7): // 7.2M Hz
				m_COMBO_SPI_CLK.SetCurSel(6); 
			break;

			case(8):  // 9M Hz
			case(9):  // 9M Hz
			case(10): // 9M Hz
				m_COMBO_SPI_CLK.SetCurSel(7); 			
			break;
		} 
	}
}

void SPI_Para_Dlg::OnBnClickedBtnSpiSet()
{
	// TODO: Add your control notification handler code here
	UCHAR  param_Clk, param_Type, param_Order, param_ss;
	UCHAR  param = 0;
	
	param_Type = m_COMBO_SPI_TYPE.GetCurSel();  

	if(param_Type == 0)
		param += 0;
	else if(param_Type == 1)
		param += 1;
	else if(param_Type == 2)
		param += 2;
	else
		param += 3;


	param_Order = m_COMBO_SPI_ORDER.GetCurSel();  

	if(param_Order)
		param = param + 4;  // MSB

	param_ss = m_COMBO_SPI_SS.GetCurSel();

	if(param_ss)
		param = param + 8;  // High Trigger

	param_Clk = m_COMBO_SPI_CLK.GetCurSel();

	
	if(m_iSpiMode != MODE_SPI_MONITOR)
	{
		switch(param_Clk)
		{
			case(0):
				param = param;		  // 1M Hz
			break;
			case(1):
				param = param | 0x10; // 2M Hz
			break;
			case(2):
				param = param | 0x20; // 3M Hz
			break;
			case(3):
				param = param | 0x30; // 4M Hz
			break;
			case(4):
				param = param | 0x40; // 4.5M Hz
			break;
			case(5):
				param = param | 0x50; // 6M Hz
			break;
			case(6):
				param = param | 0x70; // 7.2M Hz
			break;
			case(7):
				param = param | 0x90; // 9M Hz
			break;
			default:
				param = param | 0xf0; // 12M Hz
			break;
		}
	}
	else
	{
		param = param | 0xf0;
	}
	
	NuUsb.NU_SetSPIParam(m_iSpiMode, param);

	Sleep(2);
	OnOK();
	return;
}
void SPI_Para_Dlg::SPIParmSwitchLang(int LanguageIdx)
{	
	int i;
	CString text;
    HMODULE hSubDLL;
	if(LanguageIdx == 1)
	{
		i = 0;
		m_SPIParmbtns[i].SetBitmaps(IDB_BITMAP_CHT_BTN_GET, RGB(0, 255, 0)); 
		m_SPIParmbtns[i++].DrawTransparent();

		m_SPIParmbtns[i].SetBitmaps(IDB_BITMAP_CHT_BTN_SET, RGB(0, 255, 0)); 
		m_SPIParmbtns[i++].DrawTransparent();

		hSubDLL = LoadLibrary(_T("NuLang_zh-CHT.dll"));
	}
	else if(LanguageIdx == 2)
	{
		i = 0;
		m_SPIParmbtns[i].SetBitmaps(IDB_BITMAP_CHS_BTN_GET, RGB(0, 255, 0)); 
		m_SPIParmbtns[i++].DrawTransparent();

		m_SPIParmbtns[i].SetBitmaps(IDB_BITMAP_CHS_BTN_SET, RGB(0, 255, 0)); 
		m_SPIParmbtns[i++].DrawTransparent();

		hSubDLL = LoadLibrary(_T("NuLang_zh-CHS.dll"));
	}
	else
	{
		i = 0;
		m_SPIParmbtns[i].SetBitmaps(IDB_BITMAP_BTN_GET, RGB(0, 255, 0)); 
		m_SPIParmbtns[i++].DrawTransparent();

		m_SPIParmbtns[i].SetBitmaps(IDB_BITMAP_BTN_SET, RGB(0, 255, 0)); 
		m_SPIParmbtns[i++].DrawTransparent();

		hSubDLL = LoadLibrary(_T("NuLang_en-US.dll"));
	}

	if (hSubDLL)
	{
		text.LoadString(hSubDLL, IDS_CLOCK);
		GetDlgItem(IDC_STATIC_SPI_CLK)->SetWindowText(text);
		text.LoadString(hSubDLL, IDS_TYPE);
		GetDlgItem(IDC_STATIC_SPI_MODE)->SetWindowText(text);
		text.LoadString(hSubDLL, IDS_BITS);
		GetDlgItem(IDC_STATIC_SPI_BITS)->SetWindowText(text);
		text.LoadString(hSubDLL, IDS_ORDER);
		GetDlgItem(IDC_STATIC_SPI_ORDER)->SetWindowText(text);
	}
}
void SPI_Para_Dlg::OnCbnSelchangeComboSpiMode()
{
	// TODO: Add your control notification handler code here
	int idx = m_COMBO_SPI_TYPE.GetCurSel();
	int ssIdx = m_COMBO_SPI_SS.GetCurSel();
	HBITMAP hbmp;
	switch (idx)
	{
		case 0:
		{
			if(ssIdx)
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE0_SS1), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
			else
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE0_SS0), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
		}
		break;
		case 1:
		{
			if(ssIdx)
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE1_SS1), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
			else
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE1_SS0), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
		}
		break;
		case 2:
		{
			if(ssIdx)
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE2_SS1), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
			else
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE2_SS0), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
		}
		break;
		case 3:
		{
			if(ssIdx)
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE3_SS1), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
			else
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_SPI0_TYPE3_SS0), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
		}
		break;
	}
	m_SPI0_TypeIMG.SetBitmap(hbmp);
}

void SPI_Para_Dlg::OnCbnSelchangeComboSpiSs()
{
	// TODO: Add your control notification handler code here
	OnCbnSelchangeComboSpiMode();
}
