// StackedWndCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "StackedWndCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// The control responds to this message, include in all rubrics
// The wParam must contain the rubric's window handle
#define		WM_RUBRIC_WND_CLICKED_ON		( WM_APP + 04100 )

/////////////////////////////////////////////////////////////////////////////
// CStackedWndCtrl

CStackedWndCtrl::CStackedWndCtrl()
{
	m_iCurRubric = 0;
	m_iRubricHeight = 40;
	m_iCurCotent = 0;
	m_bLock = FALSE;
}

CStackedWndCtrl::~CStackedWndCtrl()
{
	for( int i = 0; i < m_arrPanes.GetSize(); i++ )
	{
		// Delete the rubric window
		m_arrPanes[i]->m_pwndRubric->DestroyWindow();

		delete m_arrPanes[i]->m_pwndRubric;
		

		
		// Delete structure
		delete m_arrPanes[i];
	}

	// Delete the content window
	m_pwndContent->DestroyWindow();
	delete m_pwndContent;

	m_arrPanes.RemoveAll();
}

int CStackedWndCtrl::AddPane( CWnd* pwndRubric)
{
	// Show First content window, Hide others
	for( int i = 1; i < m_arrPanes.GetSize(); i++ )
		if( m_arrPanes[i]->m_bOpen )
			m_arrPanes[i]->m_bOpen = FALSE;

	// Create a new pane structure
	PTDS_PANE pPane = new TDS_PANE;

	if( pPane == NULL )
	{
		AfxMessageBox( _T("Failed to add a new pane to the stack.\n\nOut of memory.") );
		return -1;
	}

	
	// Copy the pointers to the rubric and content windows
	// Also, set this pane as open
	pPane->m_pwndRubric		= pwndRubric;
	pPane->m_bOpen			= FALSE;

	// Add the new pane to the end of the stack
	int iIndex = m_arrPanes.Add( pPane );

	m_arrPanes[0]->m_bOpen = TRUE;

	// Return the index of the new pane
	return iIndex;
}

void CStackedWndCtrl::RearrangeStack()
{
    CRect rFrame;

	int tmp = m_arrPanes.GetSize();

    GetClientRect( &rFrame );

    for( int i = 0; i < m_arrPanes.GetSize(); i++ )
    {
		// Rubric windows are always visible
        m_arrPanes[i]->m_pwndRubric->SetWindowPos(  NULL,
                                                    0,
                                                    rFrame.top,
                                                    rFrame.Width(),
                                                    m_iRubricHeight,
                                                    SWP_NOZORDER | SWP_SHOWWINDOW );

		m_arrPanes[i]->m_pwndRubric->RedrawWindow();													

		// Only the content window of the flagged pane is shown
		// All others are hidden if they aren't already
        if( m_arrPanes[i]->m_bOpen )
        {
			// From the bottom of the frame, take off as many rubric
			// window's heights as there are left to display
            int iContentWndBottom = rFrame.bottom - ( ( m_arrPanes.GetSize() - i ) * m_iRubricHeight );

			m_pwndContent->SetBitmap(m_arrBMP[m_iCurCotent]);

            m_pwndContent->SetWindowPos(   NULL,
					0,
					rFrame.top + m_iRubricHeight,
					rFrame.Width(),
					iContentWndBottom - rFrame.top,
					SWP_NOZORDER | SWP_SHOWWINDOW );

			// The next rubric window will be placed right below
			// this pane's content window
            rFrame.top = iContentWndBottom;
        }

		// The top of the frame is offset by the height of a rubric window
        rFrame.top += m_iRubricHeight;
    }
	m_pwndContent->RedrawWindow();
}


BEGIN_MESSAGE_MAP(CStackedWndCtrl, CStatic)
	//{{AFX_MSG_MAP(CStackedWndCtrl)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CStackedWndCtrl message handlers

void CStackedWndCtrl::PreSubclassWindow() 
{
	// Remove the black frame and clip children to reduce flickering
	ModifyStyle( SS_BLACKFRAME, WS_CLIPCHILDREN );

	CStatic::PreSubclassWindow();
}

void CStackedWndCtrl::OnSize(UINT nType, int cx, int cy) 
{
	CStatic::OnSize(nType, cx, cy);

	RearrangeStack();
}

BOOL CStackedWndCtrl::CreateStackedWnd(int iHeight)
{
	UINT 		nIDBitmapResource;
	HBITMAP		hbmp;
	CRect rFrame;

	GetClientRect(&rFrame);
	
	// Update m_iRubricCnt here
	m_iRubricCnt = 4;
	
	// Create Rubric Windows
	for(int i = 0; i < m_iRubricCnt; i++)
	{
		switch(i)
		{
			case 0:
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMP_LEFT_SPI), IMAGE_BITMAP, rFrame.Width(), m_iRubricHeight, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
				nIDBitmapResource = IDB_BITMP_LEFT_SPI;		
				break;
			
			case 1:
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMP_LEFT_I2C), IMAGE_BITMAP, rFrame.Width(), m_iRubricHeight, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
				nIDBitmapResource = IDB_BITMP_LEFT_I2C;
				break;

			case 2:
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMP_LEFT_PWM), IMAGE_BITMAP, rFrame.Width(), m_iRubricHeight, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
				nIDBitmapResource = IDB_BITMP_LEFT_PWM;		
				break;
			
			case 3:
				hbmp =(HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMP_LEFT_ADC), IMAGE_BITMAP, rFrame.Width(), m_iRubricHeight, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
				nIDBitmapResource = IDB_BITMP_LEFT_ADC;
				break;
		}

		CStatic*	pwndRubric = new CStatic;
		pwndRubric->Create(NULL,WS_CHILD|WS_VISIBLE|SS_BITMAP,rFrame,this);
		pwndRubric->SetBitmap(LoadBitmap(AfxGetResourceHandle(), MAKEINTRESOURCE(nIDBitmapResource)));
		
		AddPane(pwndRubric);
	}

	int cWidth = rFrame.Width();
	int cHeight = rFrame.Height() - m_iRubricCnt * m_iRubricHeight;
	m_arrBMP[0] = (HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_PIN_SPI0), IMAGE_BITMAP, cWidth, cHeight, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
	m_arrBMP[1] = (HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_PIN_SPI2), IMAGE_BITMAP, cWidth, cHeight, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
	m_arrBMP[2] = (HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_PIN_I2C_A), IMAGE_BITMAP, cWidth, cHeight, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
	//m_arrBMP[3] = (HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_PIN_I2C_B), IMAGE_BITMAP, cWidth, cHeight, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
	m_arrBMP[3] = (HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_PIN_PWM_CAP), IMAGE_BITMAP, cWidth, cHeight, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
	m_arrBMP[4] = (HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_PIN_PWM_OUT), IMAGE_BITMAP, cWidth, cHeight, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );
	m_arrBMP[5] = (HBITMAP)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BITMAP_PIN_ADC), IMAGE_BITMAP, cWidth, cHeight, LR_CREATEDIBSECTION | LR_DEFAULTSIZE );	
	
	CStatic*	pContent = new CStatic;
	pContent->Create(NULL,WS_CHILD|WS_VISIBLE|SS_BITMAP,rFrame,this);
	DWORD dw = GetLastError();
	pContent->SetBitmap(m_arrBMP[m_iCurCotent] );
	pContent->GetBitmap();
			
	m_pwndContent = pContent;
	
	// Rearrange the stack
	RearrangeStack();

	return TRUE;

}

BOOL CStackedWndCtrl::SwitchContent(int iCurRubricIdx, int iCurCotentIdx)
{
	if(m_iCurCotent != iCurCotentIdx)
	{
		m_iCurCotent = iCurCotentIdx;
		m_pwndContent->SetBitmap(m_arrBMP[m_iCurCotent] );

		if(m_iCurRubric != iCurRubricIdx)
		{
			m_arrPanes[m_iCurRubric]->m_bOpen = FALSE;
			m_arrPanes[iCurRubricIdx]->m_bOpen = TRUE;
			m_iCurRubric = iCurRubricIdx;
		}

		RearrangeStack();
		return TRUE;
	}
	else
		return FALSE;
}
