/**************************************************************************//**
 * @file     NuWinUsb.cpp
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 13/12/03 11:27a $
 * @brief    NuBridge API Command 
 *
 * @note
 * Copyright (C) 2013 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include "stdafx.h"
#include "NuWinUsb.h"

// Linked libraries
#pragma comment (lib , "setupapi.lib" )
#pragma comment (lib , "winusb.lib" )

CNuWinUsb NuUsb;
OVERLAPPED overlap;

static const GUID OSR_DEVICE_INTERFACE = 

{0x1f4911f2, 0xc55d, 0x4561, {0x98, 0x7a, 0xbc, 0xfa, 0x74, 0xd5, 0xbb, 0x6d}};


CNuWinUsb::CNuWinUsb()
			:WinUsbNumber(0)
			, m_hReadEvent(CreateEvent(NULL,TRUE,FALSE,NULL))
			, m_hWriteEvent(CreateEvent(NULL,TRUE,FALSE,NULL))
			, m_ActiveDevice(0)
{

}

CNuWinUsb::~CNuWinUsb()
{
	NU_CloseHandle();
}

ULONG CNuWinUsb::NU_ReadPipe(int id, UCHAR *buf, ULONG len)
{
	ULONG nBytesRead=0;
	if(id >= WinUsbNumber || id < 0) 
		return FALSE;
	WinUsb_ReadPipe(WinUsbHandle[id].hUSBHandle, WinUsbHandle[id].pipeid.PipeInId, (unsigned char*)buf, len, &nBytesRead, 0);

	return nBytesRead;
}
BOOL CNuWinUsb::NU_AbortReadPipe(int id)
{
	if(id >= WinUsbNumber || id < 0) 
		return FALSE;
	return WinUsb_AbortPipe(WinUsbHandle[id].hUSBHandle, WinUsbHandle[id].pipeid.PipeInId);
}

BOOL CNuWinUsb::NU_WritePipe(UCHAR *buf,ULONG len)
{

	ULONG nBytesSent=0;
	if(m_ActiveDevice >= WinUsbNumber || m_ActiveDevice < 0) return FALSE;

	nBytesSent=0;

	WinUsb_WritePipe(WinUsbHandle[m_ActiveDevice].hUSBHandle, WinUsbHandle[m_ActiveDevice].pipeid.PipeOutId, buf, len, &nBytesSent, 0);

	if(nBytesSent!=len)
	{
		return FALSE;
	}
	return TRUE;
}

void CNuWinUsb::NU_CloseHandle(void)
{
	while(WinUsbNumber)
	{
		WinUsbNumber--;
		CloseHandle(WinUsbHandle[WinUsbNumber].hDeviceHandle);
		WinUsb_Free(WinUsbHandle[WinUsbNumber].hUSBHandle);
		WinUsbHandle[WinUsbNumber].hDeviceHandle=INVALID_HANDLE_VALUE;
		WinUsbHandle[WinUsbNumber].hUSBHandle=INVALID_HANDLE_VALUE;

	}
}

BOOL CNuWinUsb::EnableWinUsbDevice(void)
{
	BOOL bResult;
	bResult = GetDeviceHandle();
	if(!bResult){
		printf("GetDeviceHandle error\n");
		return bResult;
	}
	
	bResult = GetWinUSBHandle();
	if(!bResult){
		for(int i=0;i<WinUsbNumber;i++)		
			CloseHandle(WinUsbHandle[i].hDeviceHandle);
		WinUsbNumber=0;
		printf("GetWinUSBHandle error\n");
		return bResult;
	}
	
	bResult = GetUSBDeviceSpeed();
	if(!bResult){
		printf("GetUSBDeviceSpeed error\n");
		return bResult;
	}
	
	bResult = QueryDeviceEndpoints();
	if(!bResult){
		for(int i=0;i<WinUsbNumber;i++)
		{	
			CloseHandle(WinUsbHandle[i].hDeviceHandle);
			WinUsb_Free(WinUsbHandle[i].hUSBHandle);
		}
		WinUsbNumber=0;
		printf("QueryDeviceEndpoints error\n");
		return bResult;
	}
	Sleep(100);
	return bResult;
}

BOOL CNuWinUsb::GetDeviceHandle(void)
{
    if (OSR_DEVICE_INTERFACE == GUID_NULL)
    {
        return FALSE;
    }

    BOOL bResult = TRUE;
    HDEVINFO hDeviceInfo;
    SP_DEVINFO_DATA DeviceInfoData;

    SP_DEVICE_INTERFACE_DATA deviceInterfaceData;
    PSP_DEVICE_INTERFACE_DETAIL_DATA pInterfaceDetailData = NULL;

    ULONG requiredLength=0;

    LPTSTR lpDevicePath = NULL;

    DWORD index = 0;

    // Get information about all the installed devices for the specified
    // device interface class.
    hDeviceInfo = SetupDiGetClassDevs( 
        &OSR_DEVICE_INTERFACE,
        NULL, 
        NULL,
        DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);

    if (hDeviceInfo == INVALID_HANDLE_VALUE) 
    { 
        // ERROR 
        //printf("Error SetupDiGetClassDevs: %d.\n", GetLastError());
        goto done;
    }

    //Enumerate all the device interfaces in the device information set.
    DeviceInfoData.cbSize = sizeof(SP_DEVINFO_DATA);

    for (index = 0; SetupDiEnumDeviceInfo(hDeviceInfo, index, &DeviceInfoData); index++)
    {
        deviceInterfaceData.cbSize = sizeof(SP_INTERFACE_DEVICE_DATA);

        //Get information about the device interface.
        bResult = SetupDiEnumDeviceInterfaces( 
           hDeviceInfo,
           &DeviceInfoData,
           &OSR_DEVICE_INTERFACE,
           0, 
           &deviceInterfaceData);

        // Check if last item
        if (GetLastError () == ERROR_NO_MORE_ITEMS)
        {
            break;
        }

        //Check for some other error
        if (!bResult) 
        {
            //printf("Error SetupDiEnumDeviceInterfaces: %d.\n", GetLastError());
            goto done;
        }

         //Reset for this iteration
        if (lpDevicePath)
        {
            LocalFree(lpDevicePath);
        }
        if (pInterfaceDetailData)
        {
            LocalFree(pInterfaceDetailData);
        }

       //Interface data is returned in SP_DEVICE_INTERFACE_DETAIL_DATA
        //which we need to allocate, so we have to call this function twice.
        //First to get the size so that we know how much to allocate
        //Second, the actual call with the allocated buffer
        
        bResult = SetupDiGetDeviceInterfaceDetail(
            hDeviceInfo,
            &deviceInterfaceData,
            NULL, 0,
            &requiredLength,
            NULL);


        //Check for some other error
        if (!bResult) 
        {
            if ((ERROR_INSUFFICIENT_BUFFER==GetLastError()) && (requiredLength>0))
            {
                //we got the size, allocate buffer
                pInterfaceDetailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA)LocalAlloc(LPTR, requiredLength);
                
                if (!pInterfaceDetailData) 
                { 
                    // ERROR 
                    //printf("Error allocating memory for the device detail buffer.\n");
                    goto done;
                }
            }
            else
            {
                //printf("Error SetupDiEnumDeviceInterfaces: %d.\n", GetLastError());
                goto done;
            }
        }

        //get the interface detailed data
        pInterfaceDetailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

        //Now call it with the correct size and allocated buffer
        bResult = SetupDiGetDeviceInterfaceDetail(
                hDeviceInfo,
                &deviceInterfaceData,
                pInterfaceDetailData,
                requiredLength,
                NULL,
                &DeviceInfoData);
        
        //Check for some other error
        if (!bResult) 
        {
            //printf("Error SetupDiGetDeviceInterfaceDetail: %d.\n", GetLastError());
            goto done;
        }

        //copy device path
                
        //size_t nLength = strlen(pInterfaceDetailData->DevicePath) + 1; 
		size_t nLength = _tcslen(pInterfaceDetailData->DevicePath) + 1; 

		lpDevicePath = (TCHAR *) LocalAlloc (LPTR, nLength * sizeof(TCHAR));
        StringCchCopy(lpDevicePath, nLength, pInterfaceDetailData->DevicePath);
        lpDevicePath[nLength-1] = 0;
                        
        //printf("Device path:  %s\n", lpDevicePath);
		if (!lpDevicePath)
		{
			//printf("Error %d.", GetLastError());
			goto done;
		}

		//Open the device
		WinUsbHandle[index].hDeviceHandle = CreateFile (
			lpDevicePath,
			GENERIC_READ | GENERIC_WRITE,
			FILE_SHARE_READ | FILE_SHARE_WRITE,
			NULL,
			OPEN_EXISTING,
			FILE_FLAG_OVERLAPPED,
			NULL);

		if (WinUsbHandle[index].hDeviceHandle == INVALID_HANDLE_VALUE)
		{        
			//printf("Error %d.", GetLastError());
			goto done;
		}
		WinUsbHandle[index].DevicePath.Format(_T("%s"),lpDevicePath);
		WinUsbNumber=index+1;
    }


done:
    LocalFree(lpDevicePath);
    LocalFree(pInterfaceDetailData);    
    bResult = SetupDiDestroyDeviceInfoList(hDeviceInfo);
    return bResult;
}

BOOL CNuWinUsb::GetWinUSBHandle(void)
{
	int i=0;
	BOOL bResult=FALSE;
	for(i=0; i<WinUsbNumber ; i++)
	{
		if (WinUsbHandle[i].hDeviceHandle == INVALID_HANDLE_VALUE)
			return FALSE;    
		bResult = WinUsb_Initialize(WinUsbHandle[i].hDeviceHandle, &WinUsbHandle[i].hUSBHandle);
		if(!bResult)    
			return FALSE;		
	}
    return bResult;
}


BOOL CNuWinUsb::GetUSBDeviceSpeed()
{
	int i=0;
	BOOL bResult = TRUE;
	ULONG length = sizeof(UCHAR);
	UCHAR DeviceSpeed;
	for(i=0; i<WinUsbNumber ; i++)
	{
		if (!(&DeviceSpeed) || WinUsbHandle[i].hDeviceHandle==INVALID_HANDLE_VALUE)    
			return FALSE;

		bResult = WinUsb_QueryDeviceInformation(WinUsbHandle[i].hUSBHandle, DEVICE_SPEED, &length, &DeviceSpeed);
		if(!bResult)
		{
			//printf("Error getting device speed: %d.\n", GetLastError());
			return bResult;
		}
	}
	return bResult;
    //if(*pDeviceSpeed == LowSpeed)
    //if(*pDeviceSpeed == FullSpeed)
    //if(*pDeviceSpeed == HighSpeed)
}


BOOL CNuWinUsb::QueryDeviceEndpoints(void)
{
	int i=0;
	PIPE_ID* pipeid;
	BOOL bResult = TRUE;
	WINUSB_INTERFACE_HANDLE hUSBHandle;
	for(i=0; i<WinUsbNumber ; i++)
	{

		hUSBHandle=WinUsbHandle[i].hUSBHandle;
		pipeid=&(WinUsbHandle[i].pipeid);
		if (hUSBHandle==INVALID_HANDLE_VALUE)
			return FALSE;
	    

		

		USB_INTERFACE_DESCRIPTOR InterfaceDescriptor;
		ZeroMemory(&InterfaceDescriptor, sizeof(USB_INTERFACE_DESCRIPTOR));

		WINUSB_PIPE_INFORMATION  Pipe;
		ZeroMemory(&Pipe, sizeof(WINUSB_PIPE_INFORMATION));

		bResult = WinUsb_QueryInterfaceSettings(hUSBHandle, 0, &InterfaceDescriptor);

		if (bResult)
		{
			for (int index = 0; index < InterfaceDescriptor.bNumEndpoints; index++)
			{
				bResult = WinUsb_QueryPipe(hUSBHandle, 0, index, &Pipe);

				if (bResult)
				{
					if (Pipe.PipeType == UsbdPipeTypeControl)
					{
						//printf("Endpoint index: %d Pipe type: Control<0x%x> Pipe ID: 0x%x.\n", index, Pipe.PipeType, Pipe.PipeId);
					}
					if (Pipe.PipeType == UsbdPipeTypeIsochronous)
					{
						//printf("Endpoint index: %d Pipe type: Isochronous<0x%x> Pipe ID: 0x%x.\n", index, Pipe.PipeType, Pipe.PipeId);
					}
					if (Pipe.PipeType == UsbdPipeTypeBulk)
					{
						if (USB_ENDPOINT_DIRECTION_IN(Pipe.PipeId))
						{
							//printf("Endpoint index: %d Pipe type: Bulk<0x%x> Pipe ID: 0x%x.\n", index, Pipe.PipeType, Pipe.PipeId);
							pipeid->PipeInId = Pipe.PipeId;
						}
						if (USB_ENDPOINT_DIRECTION_OUT(Pipe.PipeId))
						{
							//printf("Endpoint index: %d Pipe type: Bulk<0x%x> Pipe ID: 0x%x.\n", index, Pipe.PipeType, Pipe.PipeId);
							pipeid->PipeOutId = Pipe.PipeId;
						}

					}
					if (Pipe.PipeType == UsbdPipeTypeInterrupt)
					{
						//printf("Endpoint index: %d Pipe type: Interrupt<0x%x> Pipe ID: 0x%x.\n", index, Pipe.PipeType, Pipe.PipeId);
					}
				}
				else
				{
					continue;
				}
			}
		}

	}
//done:
    return bResult;
}

BOOL CNuWinUsb::NU_SetDeviceLED(int id)
{
	WINUSB_SETUP_PACKET SetupPacket;
	SetupPacket.RequestType = WM_LED;
	SetupPacket.Request = 0;
	SetupPacket.Value = 0;
	SetupPacket.Index = 0; 
	SetupPacket.Length = 0;
	
	memset(&overlap, 0, sizeof(overlap));
	overlap.hEvent = m_hReadEvent;

	return WinUsb_ControlTransfer(WinUsbHandle[id].hUSBHandle, SetupPacket, 0, 0, NULL, &overlap);
}

BOOL CNuWinUsb::NU_SetBridgeStop(UCHAR Mode)
{
	WINUSB_SETUP_PACKET SetupPacket;
	SetupPacket.RequestType = WM_SET_DISABLE;
	SetupPacket.Request = Mode;
	SetupPacket.Value = 0;
	SetupPacket.Index = 0; 
	SetupPacket.Length = 0;

	
	memset(&overlap, 0, sizeof(overlap));
	overlap.hEvent = m_hReadEvent;
	return WinUsb_ControlTransfer(WinUsbHandle[m_ActiveDevice].hUSBHandle, SetupPacket, 0, 0, NULL, &overlap);
}

BOOL CNuWinUsb::NU_SetBridgeMode(UCHAR Mode)
{
	WINUSB_SETUP_PACKET SetupPacket;
	SetupPacket.RequestType = WM_SWITCH_MODE;
	SetupPacket.Request = Mode;
	SetupPacket.Value = 0;
	SetupPacket.Index = 0; 
	SetupPacket.Length = 0;

	
	memset(&overlap, 0, sizeof(overlap));
	overlap.hEvent = m_hReadEvent;
	return WinUsb_ControlTransfer(WinUsbHandle[m_ActiveDevice].hUSBHandle, SetupPacket, 0, 0, NULL, &overlap);
}


BOOL CNuWinUsb::NU_GetBridgeParam(UCHAR *PARM)
{
	memset(PARM, 0 , 64);
	WINUSB_SETUP_PACKET SetupPacket;
	SetupPacket.RequestType = CMD_GET_PARAM;
	SetupPacket.Request = 0;
	SetupPacket.Value = 0;
	SetupPacket.Index = 0; 
	SetupPacket.Length = 64;

	
	ULONG readByte;
	WinUsb_ControlTransfer(WinUsbHandle[m_ActiveDevice].hUSBHandle, SetupPacket, PARM, 64, &readByte, 0);
	return 1;
}

BOOL CNuWinUsb::NU_SetSPIParam(int iMode, UCHAR setting)
{
	WINUSB_SETUP_PACKET SetupPacket;
	// SetupPacket.RequestType = WM_REINIT_SPIM;
	if(MODE_SPI0_MASTER == iMode)
	{
		SetupPacket.RequestType = WM_REINIT_SPI0;
	}
	else if(MODE_SPI_MONITOR == iMode)
	{
		SetupPacket.RequestType = WM_REINIT_SPIM;
	}
	
	else if(MODE_SPI2_MASTER == iMode)
	{
		SetupPacket.RequestType = WM_REINIT_SPI2;
	}
	else
	{
		// Error
		return FALSE;
	}
	
	
	SetupPacket.Request = setting;
	SetupPacket.Value = 0;
	SetupPacket.Index = 0; 
	SetupPacket.Length = 0;

	
	memset(&overlap, 0, sizeof(overlap));
	overlap.hEvent = m_hReadEvent;
	return WinUsb_ControlTransfer(WinUsbHandle[m_ActiveDevice].hUSBHandle, SetupPacket, 0, 0, NULL, &overlap);
}

UCHAR CNuWinUsb::NU_GetSPIParam(int iMode)
{
	UCHAR PARM[64];
	NU_GetBridgeParam(PARM);
	return PARM[iMode];
}

UCHAR CNuWinUsb::NU_GetI2C0Param()
{
	UCHAR PARM[64];
	NU_GetBridgeParam(PARM);
	return PARM[MODE_I2C0_MASTER];
}
BOOL CNuWinUsb::NU_ResetFW()
{
	WINUSB_SETUP_PACKET SetupPacket;
	SetupPacket.RequestType = WM_RESET;
	SetupPacket.Request = 0;
	SetupPacket.Value = 0;
	SetupPacket.Index = 0; 

	WinUsb_ControlTransfer(WinUsbHandle[m_ActiveDevice].hUSBHandle, SetupPacket, 0, 0, NULL, 0);
	return 1;
}

BOOL CNuWinUsb::NU_SetI2C0Param(UCHAR setting)
{
	WINUSB_SETUP_PACKET SetupPacket;
	SetupPacket.RequestType = WM_REINIT_I2C0;
	SetupPacket.Request = setting;
	SetupPacket.Value = 0;
	SetupPacket.Index = 0; 
	SetupPacket.Length = 0;

	
	memset(&overlap, 0, sizeof(overlap));
	overlap.hEvent = m_hReadEvent;
	return WinUsb_ControlTransfer(WinUsbHandle[m_ActiveDevice].hUSBHandle, SetupPacket, 0, 0, NULL, &overlap);
}

BOOL CNuWinUsb::NU_Data2Bridge(UCHAR *Buf, int Size, UCHAR Type)
{
	UCHAR DataOut[64] = {0};
	DataOut[0] = Type;	// Insert 1 Byte Header in every USB Data Packet
	
	USHORT DataCnt = Size;
	int Offset = 0;
	
	while(DataCnt > 63)
	{
		memcpy(&DataOut[1], &Buf[Offset], 63);
		NU_WritePipe(DataOut, 64); 
		DataCnt -= 63;
		Offset += 63;
	}

	if(DataCnt)
	{
		memcpy(&DataOut[1], &Buf[Offset], DataCnt);
		NU_WritePipe(DataOut, DataCnt + 1); 
	}
	
	// Send Data Ready Tag after transmission complete
	DataOut[0] = 0;
	DataOut[1] = Type;
	
	NU_WritePipe(DataOut, 2);
	return 1;
}

int CNuWinUsb::NU_CheckDevice(CString szDevId)
{
	
	CString DevPath;
	
	for(int i = 0; i < WinUsbNumber; i++)
	{
		DevPath = WinUsbHandle[i].DevicePath;
		DevPath.MakeUpper();
		if (DevPath == szDevId)
			return i;
	}

	return (-1);
}

BOOL CNuWinUsb::NU_Test(void)
{
	UCHAR DataTmp[64];

	for(int i = 0; i < 64; i++)
		DataTmp[i] = i;

	DataTmp[0] = 0;     
	DataTmp[1] = 8;     
	        
	NU_WritePipe(DataTmp, 64);
	return (1);
}